/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

#ifndef __EPP_CHECKRESULT_H
#define __EPP_CHECKRESULT_H

/** @file epp_CheckResult.h
 *  @brief Definition of epp_CheckResult and epp_CheckResult_ref
 *  @author Asbjorn Steira Mikkelsen
 */

#include "epp_eppdata.h"

namespace eppobject { namespace epp {
   
   /// Class containing the true/false response of object check queries
   class epp_CheckResult {
      
    public:
      epp_boolean_ref m_avail; /**< Boolean value indicating whether an object is known to the server or not */
      epp_string_ref m_value; /**< ID of the queried object */
      epp_string_ref m_reason; /**< Server-specific text to help explain why the object is unavailable */
      epp_string_ref m_lang; /**< ID of the language of the reason text (iso639) */
      
      /// Null constructor
      epp_CheckResult() {};
      
      /// Destructor
      virtual ~epp_CheckResult() {};

      /// Constructor, using simple types when it can
      /// @note Cannot set simple types to NULL
      epp_CheckResult(const epp_boolean & _m_avail,
		      const epp_string & _m_value,
		      const epp_string & _m_reason,
		      const epp_string & _m_lang)
	{
	   //lint -e1556
	   // We know that this line is not really suspicious...
	   m_avail.ref(new epp_boolean(_m_avail));
	   //lint +e1556
	   m_value.ref(new epp_string(_m_value));
	   m_reason.ref(new epp_string(_m_reason));
	   m_lang.ref(new epp_string(_m_lang));
	};
      
      /// Constructor, using only references
      epp_CheckResult(const epp_boolean_ref & _m_avail,
		      const epp_string_ref & _m_value,
		      const epp_string_ref & _m_reason,
		      const epp_string_ref & _m_lang)
	{
	   m_avail = _m_avail;
	   m_value = _m_value;
	   m_reason = _m_reason;
	   m_lang = _m_lang;
	};
   };
   
   /// Typedef for the ref to the epp_CheckResult class
   typedef refcnt_ptr<epp_CheckResult> epp_CheckResult_ref;
   
}}

#endif
