/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

#ifndef __EPP_CONTACTADDRESS_H
#define __EPP_CONTACTADDRESS_H

/** @file epp_ContactAddress.h
 *  @brief Definition of epp_ContactAddress and epp_ContactAddress_ref
 *  @author Asbjorn Steira Mikkelsen
 */

#include "epp_contactdata.h"

namespace eppobject { namespace contact {
   
   /// Class containing elements used to describe the address of a contact
   class epp_ContactAddress {
      
    public:
      eppobject::epp::epp_string_ref m_street1; /**< Line 1 of street address */
      eppobject::epp::epp_string_ref m_street2; /**< Line 2 of street address */
      eppobject::epp::epp_string_ref m_street3; /**< Line 3 of street address */
      eppobject::epp::epp_string_ref m_city; /**< Contact city */
      eppobject::epp::epp_string_ref m_state_province; /**< Contact state/province */
      eppobject::epp::epp_string_ref m_postal_code; /**< Contact postal code */
      eppobject::epp::epp_string_ref m_country_code; /**< Contact country code (iso3166) */
      
      /// Null constructor
      epp_ContactAddress() {};
      
      /// Destructor
      virtual ~epp_ContactAddress() {};
      
      /// Constructor, using simple types when it can
      /// @note Cannot set simple types to NULL
      epp_ContactAddress(const eppobject::epp::epp_string & _m_street1,
			 const eppobject::epp::epp_string & _m_street2,
			 const eppobject::epp::epp_string & _m_street3,
			 const eppobject::epp::epp_string & _m_city,
			 const eppobject::epp::epp_string & _m_state_province,
			 const eppobject::epp::epp_string & _m_postal_code,
			 const eppobject::epp::epp_string & _m_country_code)
	{
	   m_street1.ref(new eppobject::epp::epp_string(_m_street1));
	   m_street2.ref(new eppobject::epp::epp_string(_m_street2));
	   m_street3.ref(new eppobject::epp::epp_string(_m_street3));
	   m_city.ref(new eppobject::epp::epp_string(_m_city));
	   m_state_province.ref(new eppobject::epp::epp_string(_m_state_province));
	   m_postal_code.ref(new eppobject::epp::epp_string(_m_postal_code));
	   m_country_code.ref(new eppobject::epp::epp_string(_m_country_code));
	};
      
      /// Constructor, using only references
      epp_ContactAddress(const eppobject::epp::epp_string_ref & _m_street1,
			 const eppobject::epp::epp_string_ref & _m_street2,
			 const eppobject::epp::epp_string_ref & _m_street3,
			 const eppobject::epp::epp_string_ref & _m_city,
			 const eppobject::epp::epp_string_ref & _m_state_province,
			 const eppobject::epp::epp_string_ref & _m_postal_code,
			 const eppobject::epp::epp_string_ref & _m_country_code)
	{
	   m_street1 = _m_street1;
	   m_street2 = _m_street2;
	   m_street3 = _m_street3;
	   m_city = _m_city;
	   m_state_province = _m_state_province;
	   m_postal_code = _m_postal_code;
	   m_country_code = _m_country_code;
	};
   };

   /// Typedef for the ref to the epp_ContactAddress class
   typedef refcnt_ptr<epp_ContactAddress> epp_ContactAddress_ref;
   
}}

#endif
