/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

#ifndef __EPP_CONTACTCREATEREQ_H
#define __EPP_CONTACTCREATEREQ_H

/* $Header: /cvsroot/epp-rtk/epp-rtk/c++/src/data/epp_ContactCreateReq.h,v 1.14 2003/07/18 20:21:19 tubadanm Exp $ */

/** @file epp_ContactCreateReq.h
 *  @brief Definition of epp_ContactCreateReq and epp_ContactCreateReq_ref
 *  @author Asbjorn Steira Mikkelsen
 */

#include "epp_contactdata.h"
#include "epp_GenericReq.h"
#include "epp_ContactNameAddress.h"
#include "epp_ContactPhone.h"
#include "epp_AuthInfo.h"

namespace eppobject { namespace contact {

   /// Class containing elements used in contact create requests
   /// @see epp_GenericReq for additional information
   class epp_ContactCreateReq : public epp::epp_GenericReq {
      
    public:
      eppobject::epp::epp_string_ref m_id; /**< Desired server-unique identifier for the contact to be created */
      epp_ContactNameAddress_seq_ref m_addresses; /**< Address elements, using either UTF8 or 7-bit US-ASCII character sets */
      epp_ContactNameAddress_ref m_ascii_address; /**< deprecated: Address element, using the 7-bit US-ASCII character set */
      epp_ContactNameAddress_ref m_i15d_address; /**< deprecated: Internationalized address element, using unrestricted UTF-8 */
      epp_ContactPhone_ref m_voice; /**< Voice telephone number of contact */
      epp_ContactPhone_ref m_fax; /**< Facsimile telephone number of contact */
      eppobject::epp::epp_string_ref m_email; /**< E-mail address of contact */
      eppobject::epp::epp_AuthInfo_ref m_auth_info; /**< Authorization information to be associated with contact */
      
      /// Null constructor
      epp_ContactCreateReq() {};
      
      /// Destructor
      virtual ~epp_ContactCreateReq() {}

      /// Constructor, using simple types when it can
      /// @note Cannot set simple types to NULL
      epp_ContactCreateReq(const eppobject::epp::epp_Command_ref & _m_cmd,
			   const eppobject::epp::epp_string & _m_id,
			   const epp_ContactNameAddress_ref & _m_ascii_address,
			   const epp_ContactNameAddress_ref & _m_i15d_address,
			   const epp_ContactPhone_ref & _m_voice,
			   const epp_ContactPhone_ref & _m_fax,
			   const eppobject::epp::epp_string & _m_email,
			   const eppobject::epp::epp_AuthInfo_ref & _m_auth_info)
	: epp::epp_GenericReq(_m_cmd)
	{
	   m_id.ref(new eppobject::epp::epp_string(_m_id));
	   m_ascii_address = _m_ascii_address;
	   m_i15d_address = _m_i15d_address;
	   m_voice = _m_voice;
	   m_fax = _m_fax;
	   m_email.ref(new eppobject::epp::epp_string(_m_email));
	   m_auth_info = _m_auth_info;
	};
      
      /// Constructor, using only references
      epp_ContactCreateReq(const eppobject::epp::epp_Command_ref & _m_cmd,
			   const eppobject::epp::epp_string_ref & _m_id,
			   const epp_ContactNameAddress_ref & _m_ascii_address,
			   const epp_ContactNameAddress_ref & _m_i15d_address,
			   const epp_ContactPhone_ref & _m_voice,
			   const epp_ContactPhone_ref & _m_fax,
			   const eppobject::epp::epp_string_ref & _m_email,
			   const eppobject::epp::epp_AuthInfo_ref & _m_auth_info)
	: epp::epp_GenericReq(_m_cmd)
	{
	   m_id = _m_id;
	   m_ascii_address = _m_ascii_address;
	   m_i15d_address = _m_i15d_address;
	   m_voice = _m_voice;
	   m_fax = _m_fax;
	   m_email = _m_email;
	   m_auth_info = _m_auth_info;
	};


      /// Constructor, using simple types when it can
      /// @note Cannot set simple types to NULL
      epp_ContactCreateReq(const eppobject::epp::epp_Command_ref & _m_cmd,
			   const eppobject::epp::epp_string & _m_id,
			   const epp_ContactNameAddress_seq_ref & _m_addresses,
			   const epp_ContactPhone_ref & _m_voice,
			   const epp_ContactPhone_ref & _m_fax,
			   const eppobject::epp::epp_string & _m_email,
			   const eppobject::epp::epp_AuthInfo_ref & _m_auth_info)
	: epp::epp_GenericReq(_m_cmd)
	{
	   m_id.ref(new eppobject::epp::epp_string(_m_id));
	   m_addresses = _m_addresses;
	   m_voice = _m_voice;
	   m_fax = _m_fax;
	   m_email.ref(new eppobject::epp::epp_string(_m_email));
	   m_auth_info = _m_auth_info;
	};
      
      /// Constructor, using only references
      epp_ContactCreateReq(const eppobject::epp::epp_Command_ref & _m_cmd,
			   const eppobject::epp::epp_string_ref & _m_id,
			   const epp_ContactNameAddress_seq_ref & _m_addresses,
			   const epp_ContactPhone_ref & _m_voice,
			   const epp_ContactPhone_ref & _m_fax,
			   const eppobject::epp::epp_string_ref & _m_email,
			   const eppobject::epp::epp_AuthInfo_ref & _m_auth_info)
	: epp::epp_GenericReq(_m_cmd)
	{
	   m_id = _m_id;
	   m_addresses = _m_addresses;
	   m_voice = _m_voice;
	   m_fax = _m_fax;
	   m_email = _m_email;
	   m_auth_info = _m_auth_info;
	};


   };
   
   /// Typedef for the ref to the epp_ContactCreateReq class
   typedef refcnt_ptr<epp_ContactCreateReq> epp_ContactCreateReq_ref;
   
}}

#endif
