/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

#ifndef __EPP_CONTACTUPDATECHANGE_H
#define __EPP_CONTACTUPDATECHANGE_H

/** @file epp_ContactUpdateChange.h
 *  @brief Definition of epp_ContactUpdateChange and epp_ContactUpdateChange_ref
 *  @author Asbjorn Steira Mikkelsen
 */

#include "epp_contactdata.h"
#include "epp_ContactNameAddress.h"
#include "epp_ContactPhone.h"
#include "epp_AuthInfo.h"

namespace eppobject { namespace contact {
   
   /// Class containing elements used in the change part of contact update requests
   class epp_ContactUpdateChange {
      
    public:
      epp_ContactNameAddress_seq_ref m_addresses; /**< Address elements, using either UTF8 or 7-bit US-ASCII character sets */
      epp_ContactNameAddress_ref m_ascii_address; /**< DEPRECATED Address element, using the 7-bit US-ASCII character set */
      epp_ContactNameAddress_ref m_i15d_address; /**< DEPRECATED Internationalized address element, using unrestricted UTF-8 */
      epp_ContactPhone_ref m_voice; /**< Voice telephone number of contact */
      epp_ContactPhone_ref m_fax; /**< Facsimile telephone number of contact */
      eppobject::epp::epp_string_ref m_email; /**< E-mail address of contact */
      eppobject::epp::epp_AuthInfo_ref m_auth_info; /**< Authorization information to be associated with contact */
      
      /// Null constructor
      epp_ContactUpdateChange() {};

      /// Destructor
      virtual ~epp_ContactUpdateChange() {};
      
      /// Constructor, using simple types when it can
      /// @note Cannot set simple types to NULL
      /// @deprecated
      epp_ContactUpdateChange(const epp_ContactNameAddress_ref & _m_ascii_address,
			      const epp_ContactNameAddress_ref & _m_i15d_address,
			      const epp_ContactPhone_ref & _m_voice,
			      const epp_ContactPhone_ref & _m_fax,
			      const eppobject::epp::epp_string & _m_email,
			      const eppobject::epp::epp_AuthInfo_ref & _m_auth_info)
	{
	   m_ascii_address = _m_ascii_address;
           if ( _m_ascii_address != NULL ) {
              epp_ContactNameAddress i(*_m_ascii_address);
              i.m_type.ref(new epp_ContactPostalInfoType(INT));
              m_addresses->push_back(i);
           }
	   m_i15d_address = _m_i15d_address;
           if ( _m_i15d_address != NULL ) {
              epp_ContactNameAddress i(*_m_i15d_address);
              i.m_type.ref(new epp_ContactPostalInfoType(INT));
              m_addresses->push_back(i);
           }
	   m_voice = _m_voice;
	   m_fax = _m_fax;
	   m_email.ref(new eppobject::epp::epp_string(_m_email));
	   m_auth_info = _m_auth_info;
	};
      
      /// Constructor, using only references
      /// @deprecated
      epp_ContactUpdateChange(const epp_ContactNameAddress_ref & _m_ascii_address,
			      const epp_ContactNameAddress_ref & _m_i15d_address,
			      const epp_ContactPhone_ref & _m_voice,
			      const epp_ContactPhone_ref & _m_fax,
			      const eppobject::epp::epp_string_ref & _m_email,
			      const eppobject::epp::epp_AuthInfo_ref & _m_auth_info)
	{
	   m_ascii_address = _m_ascii_address;
           if ( _m_ascii_address != NULL ) {
              epp_ContactNameAddress i(*_m_ascii_address);
              i.m_type.ref(new epp_ContactPostalInfoType(INT));
              m_addresses->push_back(i);
           }
	   m_i15d_address = _m_i15d_address;
           if ( _m_i15d_address != NULL ) {
              epp_ContactNameAddress i(*_m_i15d_address);
              i.m_type.ref(new epp_ContactPostalInfoType(INT));
              m_addresses->push_back(i);
           }
	   m_voice = _m_voice;
	   m_fax = _m_fax;
	   m_email = _m_email;
	   m_auth_info = _m_auth_info;
	};

      /// Constructor, using simple types when it can
      /// @note Cannot set simple types to NULL
      epp_ContactUpdateChange(const epp_ContactNameAddress_seq_ref & _m_addresses,
			      const epp_ContactPhone_ref & _m_voice,
			      const epp_ContactPhone_ref & _m_fax,
			      const eppobject::epp::epp_string & _m_email,
			      const eppobject::epp::epp_AuthInfo_ref & _m_auth_info)
	{
	   m_addresses = _m_addresses;
           if ( m_addresses != NULL ) {
              for ( epp_ContactNameAddress_seq::iterator i = m_addresses->begin();
                    i != m_addresses->end();
                    i++ ) {
                 if ( *i->m_type == LOC ) {
                    m_i15d_address.ref(new epp_ContactNameAddress(*i));
                 } else {
                    m_ascii_address.ref(new epp_ContactNameAddress(*i));
                 }
              }
           }
	   m_voice = _m_voice;
	   m_fax = _m_fax;
	   m_email.ref(new eppobject::epp::epp_string(_m_email));
	   m_auth_info = _m_auth_info;
	};
      
      /// Constructor, using only references
      epp_ContactUpdateChange(const epp_ContactNameAddress_seq_ref & _m_addresses,
			      const epp_ContactPhone_ref & _m_voice,
			      const epp_ContactPhone_ref & _m_fax,
			      const eppobject::epp::epp_string_ref & _m_email,
			      const eppobject::epp::epp_AuthInfo_ref & _m_auth_info)
	{
	   m_addresses = _m_addresses;
           if ( m_addresses != NULL ) {
              for ( epp_ContactNameAddress_seq::iterator i = m_addresses->begin();
                    i != m_addresses->end();
                    i++ ) {
                 if ( *i->m_type == LOC ) {
                    m_i15d_address.ref(new epp_ContactNameAddress(*i));
                 } else {
                    m_ascii_address.ref(new epp_ContactNameAddress(*i));
                 }
              }
           }
	   m_voice = _m_voice;
	   m_fax = _m_fax;
	   m_email = _m_email;
	   m_auth_info = _m_auth_info;
	};
   };

   /// Typedef for the ref to the epp_ContactUpdateChange class
   typedef refcnt_ptr<epp_ContactUpdateChange> epp_ContactUpdateChange_ref;
   
}}

#endif
