/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

#ifndef __EPP_CONTACTUPDATEREQ_H
#define __EPP_CONTACTUPDATEREQ_H

/** @file epp_ContactUpdateReq.h
 *  @brief Definition of epp_ContactUpdateReq and epp_ContactUpdateReq_ref
 *  @author Asbjorn Steira Mikkelsen
 */

#include "epp_contactdata.h"
#include "epp_GenericReq.h"
#include "epp_ContactUpdateAddRemove.h"
#include "epp_ContactUpdateChange.h"

namespace eppobject { namespace contact {

   /// Class containing elements used in contact update requests
   /// @see epp_GenericReq for additional information
   class epp_ContactUpdateReq : public epp::epp_GenericReq {

    public:
      eppobject::epp::epp_string_ref m_id; /**< Server-unique ID of the contact object to be updated */
      epp_ContactUpdateAddRemove_ref m_add; /**< Attribute values to be added to the object */
      epp_ContactUpdateAddRemove_ref m_remove; /**< Attribute values to be removed from the object */
      epp_ContactUpdateChange_ref m_change; /**< Object attribute values to be changed */
      
      /// Null constructor
      epp_ContactUpdateReq() {};

      /// Destructor
      virtual ~epp_ContactUpdateReq() {};
      
      /// Constructor, using simple types when it can
      /// @note Cannot set simple types to NULL
      epp_ContactUpdateReq(const eppobject::epp::epp_Command_ref & _m_cmd,
			   const eppobject::epp::epp_string & _m_id,
			   const epp_ContactUpdateAddRemove_ref & _m_add,
			   const epp_ContactUpdateAddRemove_ref & _m_remove,
			   const epp_ContactUpdateChange_ref & _m_change)
	: epp::epp_GenericReq(_m_cmd)
	{
	   m_id.ref(new eppobject::epp::epp_string(_m_id));
	   m_add = _m_add;
	   m_remove = _m_remove;
	   m_change = _m_change;
	};
      
      /// Constructor, using only references
      epp_ContactUpdateReq(const eppobject::epp::epp_Command_ref & _m_cmd,
			   const eppobject::epp::epp_string_ref & _m_id,
			   const epp_ContactUpdateAddRemove_ref & _m_add,
			   const epp_ContactUpdateAddRemove_ref & _m_remove,
			   const epp_ContactUpdateChange_ref & _m_change)
	: epp::epp_GenericReq(_m_cmd)
	{
	   m_id = _m_id;
	   m_add = _m_add;
	   m_remove = _m_remove;
	   m_change = _m_change;
	};
   };

   /// Typedef for the ref to the epp_ContactUpdateReq class
   typedef refcnt_ptr<epp_ContactUpdateReq> epp_ContactUpdateReq_ref;
   
}}

#endif
