/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

/* $Header: /cvsroot/epp-rtk/epp-rtk/c++/src/data/epp_DataCollectionPolicy.cc,v 1.5 2005/09/13 12:44:08 jamesfgould Exp $ */

/** @file epp_DataCollectionPolicy.cc
 *  @brief Definition of epp_DataCollectionPolicy and epp_DataCollectionPolicy_ref
 *  @author Daniel Manley
 */

#include "epp_DataCollectionPolicy.h"

using namespace eppobject::epp;
using namespace std;

ostream& eppobject::epp::operator<<(ostream &stream, const epp_DataCollectionPolicy & dcp) {

    stream << "epp_DataCollectionPolicy: "<<dcp.m_access<<" "<<dcp.m_statements;
	
	// JG - Added check since m_expiry is optional
	if (dcp.m_expiry != NULL) {
		stream << dcp.m_expiry;
	}
	else {
		stream << "m_expiry []";
	}

    return stream;

}

ostream& eppobject::epp::operator<<(ostream &stream, const epp_DataCollectionPolicy_ref & dcp) {

    stream << *dcp;

    return stream;

}

ostream& eppobject::epp::operator<<(ostream &stream, const epp_dcpAccessType_ref & access) {

   stream << "m_access [";
   if(access != NULL) {
      switch(*access) {

       case ALL: stream << "all"; break;
       case NONE_ACCESS: stream << "none"; break;
       case NULL_ACCESS: stream << "null"; break;
       case OTHER: stream << "other"; break;
       case PERSONAL: stream << "personal"; break;
       case PERSONAL_AND_OTHER: stream << "personalAndOther"; break;
       default: stream << "**UNKNOWN**";

      }
   }

   return stream << "]";

}

ostream& eppobject::epp::operator<<(ostream &stream, const epp_dcpStatement_seq_ref & statements) {

   stream << "m_statements [";
   
   if(statements != NULL)
     {
	for(epp_dcpStatement_seq::iterator j = statements->begin(); j != statements->end(); j++)
	  {
             if ( j != statements->begin() ) stream << ",";
	     
             stream << "m_purposes [";
              
             if(j->m_purposes != NULL)
	       {
		  string purposes = "[";
		  for(epp_dcpPurpose_seq::iterator i = j->m_purposes->begin();
		      i != j->m_purposes->end(); i++) {

                     if ( i != j->m_purposes->begin() ) purposes += ",";
		     switch(*i) {
		      case ADMIN_PURPOSE: purposes += "admin"; break;
		      case CONTACT: purposes += "contact"; break;
		      case OTHER_PURPOSE: purposes += "other"; break;
		      case PROV: purposes += "prov,"; break;
		      default: purposes += "**UNKNOWN**";
		     }

		  } // for

		  purposes += "]";
                  stream << purposes;
	       } // if "m_purposes"

             stream << "] m_recipient [";
             
	     if(j->m_recipients != NULL)
	       {
		  string recipients = "";
		  for(epp_dcpRecipient_seq::iterator i = j->m_recipients->begin();
		      i != j->m_recipients->end(); i++) {

                     if ( i != j->m_recipients->begin() ) recipients += ",";
		     switch(*i->m_type) {
		      case OTHER_RECIPIENT: recipients += "other"; break;
		      case OURS:
				  if (i->m_rec_desc != NULL) {
					  recipients += "ours{"+*i->m_rec_desc+"}";
				  }
				  else {
					  recipients += "ours";
				  }
				  break;
		      case PUBLIK: recipients += "public"; break;
		      case SAME: recipients += "same"; break;
		      case UNRELATED: recipients += "unrelated"; break;
		      default: recipients += "**UNKNOWN**";
		     }
		  } // for

		  stream << recipients;
	       } // if "m_recipients"
             stream << "] m_retention [";

	     if(j->m_retention != NULL)
	       {
		  string retention;
		  switch(*j->m_retention) {
		   case BUSINESS: retention = "business"; break;
		   case INDEFINITE: retention = "indefinite"; break;
		   case LEGAL: retention = "legal"; break;
		   case NONE: retention = "none"; break;
		   case STATED: retention = "stated"; break;
		   default: retention = "**UNKNOWN**";
		  }

		  stream << retention;
	       } // if "retentions"

             stream << "]";
             
	  }
     } // if "m_statement"
   
   
   stream << "]";
   
   return stream;

}


ostream& eppobject::epp::operator<<(ostream &stream, const epp_dcpExpiry_ref & expiry) {

   stream << "m_expiry [";
   
   if ( expiry != NULL ) {
      stream << "m_type [";
      if(expiry->m_type != NULL) {
         switch(*expiry->m_type) {

          case ABSOLUTE: stream << "absolute"; break;
          case RELATIVE: stream << "relative"; break;
          default: stream << "**UNKNOWN**";

         }
      }

      stream << "] m_value ["<<*expiry->m_value<<"]";

   }
   
   stream << "]";
   return stream;

}

