/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

#ifndef __EPP_DATACOLLECTIONPOLICY_H
#define __EPP_DATACOLLECTIONPOLICY_H

/* $Header: /cvsroot/epp-rtk/epp-rtk/c++/src/data/epp_DataCollectionPolicy.h,v 1.7 2003/07/18 20:21:19 tubadanm Exp $ */

/** @file epp_DataCollectionPolicy.h
 *  @brief Definition of epp_DataCollectionPolicy and epp_DataCollectionPolicy_ref
 *  @author Asbjorn Steira Mikkelsen
 */

#include <iostream>
#include "epp_eppdata.h"
#include "epp_dcpStatement.h"
#include "epp_dcpExpiry.h"

namespace eppobject { namespace epp {

   /// Class containing elements used to describe the server's policy for data
   /// collection and management
   class epp_DataCollectionPolicy {
      
    public:
      epp_dcpAccessType_ref m_access; /**< Describes access provided by the server to data received from the client */
      epp_dcpStatement_seq_ref m_statements; /**< Describes data collection purposes, data recipients, and data retention */
      epp_dcpExpiry_ref m_expiry; /**< The date of expiration of the DCP */
      
      /// Null constructor
      epp_DataCollectionPolicy() {};
      
      /// Destructor
      virtual ~epp_DataCollectionPolicy() {}
      
      /// Constructor, using only references
      epp_DataCollectionPolicy(const epp_dcpAccessType_ref & _m_access,
			       const epp_dcpStatement_seq_ref & _m_statements,
                               const epp_dcpExpiry_ref & _m_expiry)
	{
	   m_access = _m_access;
	   m_statements = _m_statements;
           m_expiry = _m_expiry;
	};

       friend std::ostream& operator<<(std::ostream &stream, const epp_DataCollectionPolicy & dcp);

   };
   
   /// Typedef for the ref to the epp_DataCollectionPolicy class
   typedef refcnt_ptr<epp_DataCollectionPolicy> epp_DataCollectionPolicy_ref;
   std::ostream& operator<<(std::ostream &stream, const epp_DataCollectionPolicy_ref & dcp);
   std::ostream& operator<<(std::ostream &stream, const epp_dcpAccessType_ref & access);
   std::ostream& operator<<(std::ostream &stream, const epp_dcpStatement_seq_ref & statements);
   std::ostream& operator<<(std::ostream &stream, const epp_dcpExpiry_ref & expiry);

}}

#endif
