/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

#ifndef __EPP_DOMAINCONTACT_H
#define __EPP_DOMAINCONTACT_H

/** @file epp_DomainContact.h
 *  @brief Definition of epp_DomainContact and epp_DomainContact_ref
 *  @author Asbjorn Steira Mikkelsen
 */

#include "epp_domaindata.h"

namespace eppobject { namespace domain {
   
   /// Class containing information about the human or organizational
   /// social information object associated with a domain object
   class epp_DomainContact {
      
    public:
      epp_DomainContactType_ref m_type; /**< Type of domain contact */
      eppobject::epp::epp_string_ref m_id; /**< ID of domain contact */
      
      /// Null constructor
      epp_DomainContact() {};
      
      /// Destructor
      virtual ~epp_DomainContact() {};

      /// Constructor, using simple types when it can
      /// @note Cannot set simple types to NULL
      epp_DomainContact(const epp_DomainContactType & _m_type,
			const eppobject::epp::epp_string & _m_id)
	{
	   m_type.ref(new epp_DomainContactType(_m_type));
	   m_id.ref(new eppobject::epp::epp_string(_m_id));
	};
      
      /// Constructor, using only references
      epp_DomainContact(const epp_DomainContactType_ref & _m_type,
			const eppobject::epp::epp_string_ref & _m_id)
	{
	   m_type = _m_type;
	   m_id = _m_id;
	};
   };

   /// Typedef for the ref to the epp_DomainContact class
   typedef refcnt_ptr<epp_DomainContact> epp_DomainContact_ref;
   
}}

#endif
