/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/


/** @file epp_DomainPanData.cc
 *  @brief Additional definitions for epp_DomainPanData
 *  @author Asbjorn Steira Mikkelsen
 *  @see epp_DomainPanData.h
 */

#include <domtools/dom_wrapper.h>
#include "epp_DomainPanData.h"
#include "epp_domainXMLbase.h"
#include "epp_XMLException.h"

using namespace std;
using namespace eppobject::domain;
using namespace eppobject::epp;
using namespace domtools;

epp_PollResData_ref epp_DomainPanData::createPollResData(const dom_ptr & panNode)
{
   epp_DomainPanData_ref domt(new epp_DomainPanData());
   
   if(!panNode->isNull()) {
      dom_ptr DPDChildNode = dom_ptr(panNode->getFirstChildElement());
      
      string elemPrefix = panNode->getPrefix();
      
      while(!DPDChildNode->isNull()) {
	 string nodeName = DPDChildNode->getNodeName();
	 
	 if(nodeName == string(elemPrefix + ":name")) {
	    domt->m_name.ref(new epp_string(DPDChildNode->getNodeData()));
//	    cout << "name: " << *domt->m_name << endl;

	   string result = DPDChildNode->getAttribute("paResult");
//	   cout << "result: " << result << endl;

	   if(result == "1") domt->m_result.ref(new epp_boolean(true));
	   else if(result == "0") domt->m_result.ref(new epp_boolean(false));
	   else throw epp_XMLException(__FILE__, __LINE__, "Empty or unknown result-attribute in domain:panData");

	 } // if "name"
	 
	 else if(nodeName == string(elemPrefix + ":paTRID")) {

	     domt->m_trid.ref(new epp_TransID());

	     dom_ptr trIDChildNode = dom_ptr(DPDChildNode->getFirstChildElement());

	     while(!trIDChildNode->isNull()) {
	        string trIDNodeName = trIDChildNode->getNodeName();

	        if(trIDNodeName == "svTRID") {
	           domt->m_trid->m_server_trid.ref(new epp_trid(trIDChildNode->getNodeData()));
//	           cout << "svTRID: " << *res->m_trans_id->m_server_trid << endl;
	        }
	        else if(trIDNodeName == "clTRID") {
	           domt->m_trid->m_client_trid.ref(new epp_trid(trIDChildNode->getNodeData()));
//	           cout << "clTRID: " << *res->m_trans_id->m_client_trid << endl;
	        }

	        trIDChildNode = dom_ptr(trIDChildNode->getNextSiblingElement());
	     } // while "trIDChildNode"

	 } // if "paTRID"
	 
	 else if(nodeName == string(elemPrefix + ":paDate")) {
	    domt->m_date.ref(new epp_datetime(DPDChildNode->getNodeData()));
//	    cout << "paDate: " << *domt->m_expiration_date << endl;
	 } // if "paDate"
	 
	 DPDChildNode = dom_ptr(DPDChildNode->getNextSiblingElement());
      } // while "DTChildNode"
      
   } // if "panNode"
   
   return domt;
   
} // createPollResData
