/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

#ifndef __EPP_DOMAINUPDATEADDREMOVE_H
#define __EPP_DOMAINUPDATEADDREMOVE_H

/** @file epp_DomainUpdateAddRemove.h
 *  @brief Definition of epp_DomainUpdateAddRemove and epp_DomainUpdateAddRemove_ref
 *  @author Asbjorn Steira Mikkelsen
 */

#include "epp_domaindata.h"
#include "epp_DomainContact.h"
#include "epp_DomainStatus.h"

namespace eppobject { namespace domain {
   
   /// Class containing elements used in add/remove part of domain update requests
   class epp_DomainUpdateAddRemove {
      
    public:
      eppobject::epp::epp_string_seq_ref m_name_servers; /**< Fully qualified names of known host objects to
							  * provide resolution services for the domain */
      epp_domain_contact_seq_ref m_contacts; /**< IDs for other contact objects (to be) associated with the
					      * domain object */
      epp_domain_status_seq_ref m_status; /**< Status of the domain object */
      
      /// Null constructor
      epp_DomainUpdateAddRemove() {};
      
      /// Destructor
      virtual ~epp_DomainUpdateAddRemove() {};
      
      /// Constructor, using only references
      epp_DomainUpdateAddRemove(const eppobject::epp::epp_string_seq_ref & _m_name_servers,
				const epp_domain_contact_seq_ref & _m_contacts,
				const epp_domain_status_seq_ref & _m_status)
	{
	   m_name_servers = _m_name_servers;
	   m_contacts = _m_contacts;
	   m_status = _m_status;
	};
   };
   
   /// Typedef for the ref to the epp_DomainUpdateAddRemove class
   typedef refcnt_ptr<epp_DomainUpdateAddRemove> epp_DomainUpdateAddRemove_ref;
   
}}

#endif
