/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

#ifndef __EPP_DOMAINUPDATECHANGE_H
#define __EPP_DOMAINUPDATECHANGE_H

/** @file epp_DomainUpdateChange.h
 *  @brief Definition of epp_DomainUpdateChange and epp_DomainUpdateChange_ref
 *  @author Asbjorn Steira Mikkelsen
 */

#include "epp_domaindata.h"
#include "epp_AuthInfo.h"

namespace eppobject { namespace domain {
   
   /// Class containing elements used in the change part of domain update requests
   class epp_DomainUpdateChange {
      
    public:
      eppobject::epp::epp_string_ref m_registrant; /**< ID for the contact object to be associated with the
						    * domain object as the object registrant */
      eppobject::epp::epp_AuthInfo_ref m_auth_info; /**< Authorization information to be associated with domain */
      
      /// Null constructor
      epp_DomainUpdateChange() {};
      
      /// Destructor
      virtual ~epp_DomainUpdateChange() {};
      
      /// Constructor, using simple types when it can
      /// @note Cannot set simple types to NULL
      epp_DomainUpdateChange(const eppobject::epp::epp_string & _m_registrant,
			     const eppobject::epp::epp_AuthInfo_ref & _m_auth_info)
	{
	   m_registrant.ref(new eppobject::epp::epp_string(_m_registrant));
	   m_auth_info = _m_auth_info;
	};
      
      /// Constructor, using only references
      epp_DomainUpdateChange(const eppobject::epp::epp_string_ref & _m_registrant,
			     const eppobject::epp::epp_AuthInfo_ref & _m_auth_info)
	{
	   m_registrant = _m_registrant;
	   m_auth_info = _m_auth_info;
	};
   };

   /// Typedef for the ref to the epp_DomainUpdateChange class
   typedef refcnt_ptr<epp_DomainUpdateChange> epp_DomainUpdateChange_ref;
   
}}

#endif
