/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

#ifndef __EPP_EXCEPTION_H
#define __EPP_EXCEPTION_H

/* $Header: /cvsroot/epp-rtk/epp-rtk/c++/src/data/epp_Exception.h,v 1.13 2003/07/18 20:21:19 tubadanm Exp $ */

/** @file epp_Exception.h
 *  @brief Definition of epp_Exception
 *  @author Asbjorn Steira Mikkelsen
 */

#include "epp_eppdata.h"
#include "epp_Result.h"
#include "epp_TransID.h"
#include <exception>
#include <cstdio>


namespace eppobject { namespace epp {
   
   /// Exception class containing information about errors returned from the EPP server
   /// or, if in the range 2600-2699, communication errors
   /// @see epp_Session for more information about error codes
   class epp_Exception : public std::exception {
      
    public:
      epp_result_seq_ref m_details; /**< Object describing the encountered error(s) */
      epp_TransID_ref m_trans_id; /**< Transaction identifier assigned by the server to the command */
      epp_xml_string_seq_ref m_ext_strings; /**< Extension elements used for server-defined response extensions */
      
      /// Null constructor
      epp_Exception() {};

      /// Destructor
      virtual ~epp_Exception() throw() {};
      
      /// Constructor with error data
      epp_Exception(const epp_result_seq_ref & _m_details,
		    const epp_TransID_ref & _m_trans_id,
		    const epp_xml_string_seq_ref & _m_ext_strings = epp_xml_string_seq_ref(new epp_xml_string_seq))
	{
	   m_details = _m_details;
	   m_trans_id = _m_trans_id;
	   m_ext_strings = _m_ext_strings;
	};
      
      /// Returns the error details
      /// @deprecated Only returns #epp_result_seq_ref. Access the data directly instead.
      virtual epp_result_seq_ref getDetails() const
	{
	   return m_details;
	}
      
   };   

   std::ostream& operator<<(std::ostream &stream, const epp_Exception & exception);

}}

#endif
