/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

#ifndef __EPP_EXT_MESSAGE_H
#define __EPP_EXT_MESSAGE_H

/* $Header: /cvsroot/epp-rtk/epp-rtk/c++/src/data/epp_ExtMessage.h,v 1.3 2003/07/18 20:21:19 tubadanm Exp $ */

/** @file epp_ExtMessage.h
 *  @brief Definition of epp_ExtMessage and epp_ExtMessage_ref
 *  @author Daniel Manley
 */

#include "epp_eppdata.h"
#include "epp_ResultValue.h"

namespace eppobject { namespace epp {
   
   /// Class containing elements used to describe success/failure of
   /// a command in server responses
   class epp_ExtMessage {
      
    public:
      
      epp_string_ref m_lang; /**< Text language of the reason message (iso639) */
      epp_string_ref m_value; /**< The response message */
      
      /// Null constructor
      epp_ExtMessage() {};
      
      /// Destructor
      virtual ~epp_ExtMessage() {};
      
      /// Constructor, using simple types when it can
      /// @note Cannot set simple types to NULL
      epp_ExtMessage(const epp_string & _m_lang,
		     const epp_string & _m_value)
	{
	   m_lang.ref(new epp_string(_m_lang));
	   m_value.ref(new epp_string(_m_value));
	};
      
      /// Constructor, using only references
      epp_ExtMessage(const epp_string_ref & _m_lang,
  		     const epp_string_ref & _m_value)
	{
	   m_lang = _m_lang;
	   m_value = _m_value;
	};
   };
   
   /// Typedef for the ref to the epp_ExtMessage class
   typedef refcnt_ptr<epp_ExtMessage> epp_ExtMessage_ref;
   std::ostream& operator<<(std::ostream &stream, const epp_ExtMessage_ref & extmsg);
   std::ostream& operator<<(std::ostream &stream, const epp_ExtMessage & extmsg);
   
}}

#endif
