/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

#ifndef __EPP_HOSTINFORSP_H
#define __EPP_HOSTINFORSP_H

/** @file epp_HostInfoRsp.h
 *  @brief Definition of epp_HostInfoRsp and epp_HostInfoRsp_ref
 *  @author Asbjorn Steira Mikkelsen
 */

#include "epp_hostdata.h"
#include "epp_GenericRsp.h"
#include "epp_HostStatus.h"
#include "epp_HostAddress.h"

namespace eppobject { namespace host {
   
   /// Class containing elements used in host info responses
   /// @see epp_GenericRsp for additional information
   class epp_HostInfoRsp : public epp::epp_GenericRsp {
      
    public:
      eppobject::epp::epp_string_ref m_name; /**< Fully qualified name of the host object */
      eppobject::epp::epp_roid_ref m_roid; /**< Repository Object IDentifier assigned to the host */
      epp_host_status_seq_ref m_status; /**< Status of the host object */
      epp_host_address_seq_ref m_addresses; /**< IP addresses associated with the host */
      eppobject::epp::epp_string_ref m_client_id; /**< Identifier of the sponsoring client */
      eppobject::epp::epp_string_ref m_created_by; /**< Identifier of the client that created the host */
      eppobject::epp::epp_datetime_ref m_created_date; /**< Date and time of host object creation */
      eppobject::epp::epp_string_ref m_updated_by; /**< Identifier of the client that last updated the host */
      eppobject::epp::epp_datetime_ref m_updated_date; /**< Date and time of the most recent host modification */
      eppobject::epp::epp_datetime_ref m_transfer_date; /**< Date and time of the most recent successful host transfer */
      
      /// Null constructor
      epp_HostInfoRsp() {};

      /// Destructor
      virtual ~epp_HostInfoRsp() {}
      
      /// Constructor, using simple types when it can
      /// @note Cannot set simple types to NULL
      epp_HostInfoRsp(const eppobject::epp::epp_Response_ref & _m_rsp,
		      const eppobject::epp::epp_string & _m_name,
		      const eppobject::epp::epp_roid & _m_roid,
		      const epp_host_status_seq_ref & _m_status,
		      const epp_host_address_seq_ref & _m_addresses,
		      const eppobject::epp::epp_string & _m_client_id,
		      const eppobject::epp::epp_string & _m_created_by,
		      const eppobject::epp::epp_datetime & _m_created_date,
		      const eppobject::epp::epp_string & _m_updated_by,
		      const eppobject::epp::epp_datetime & _m_updated_date,
		      const eppobject::epp::epp_datetime & _m_transfer_date)
	: epp::epp_GenericRsp(_m_rsp)
	{
	   m_name.ref(new eppobject::epp::epp_string(_m_name));
	   m_roid.ref(new eppobject::epp::epp_roid(_m_roid));
	   m_status = _m_status;
	   m_addresses = _m_addresses;
	   m_client_id.ref(new eppobject::epp::epp_string(_m_client_id));
	   m_created_by.ref(new eppobject::epp::epp_string(_m_created_by));
	   m_created_date.ref(new eppobject::epp::epp_datetime(_m_created_date));
	   m_updated_by.ref(new eppobject::epp::epp_string(_m_updated_by));
	   m_updated_date.ref(new eppobject::epp::epp_datetime(_m_updated_date));
	   m_transfer_date.ref(new eppobject::epp::epp_datetime(_m_transfer_date));
	};
      
      /// Constructor, using only references
      epp_HostInfoRsp(const eppobject::epp::epp_Response_ref & _m_rsp,
		      const eppobject::epp::epp_string_ref & _m_name,
		      const eppobject::epp::epp_roid_ref & _m_roid,
		      const epp_host_status_seq_ref & _m_status,
		      const epp_host_address_seq_ref & _m_addresses,
		      const eppobject::epp::epp_string_ref & _m_client_id,
		      const eppobject::epp::epp_string_ref & _m_created_by,
		      const eppobject::epp::epp_datetime_ref & _m_created_date,
		      const eppobject::epp::epp_string_ref & _m_updated_by,
		      const eppobject::epp::epp_datetime_ref & _m_updated_date,
		      const eppobject::epp::epp_datetime_ref & _m_transfer_date)
	: epp::epp_GenericRsp(_m_rsp)
	{
	   m_name = _m_name;
	   m_roid = _m_roid;
	   m_status = _m_status;
	   m_addresses = _m_addresses;
	   m_client_id = _m_client_id;
	   m_created_by = _m_created_by;
	   m_created_date = _m_created_date;
	   m_updated_by = _m_updated_by;
	   m_updated_date = _m_updated_date;
	   m_transfer_date = _m_transfer_date;
	};
   };

   /// Typedef for the ref to the epp_HostInfoRsp class
   typedef refcnt_ptr<epp_HostInfoRsp> epp_HostInfoRsp_ref;
   
}}

#endif
