/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

#ifndef __EPP_HOSTUPDATEREQ_H
#define __EPP_HOSTUPDATEREQ_H

/** @file epp_HostUpdateReq.h
 *  @brief Definition of epp_HostUpdateReq and epp_HostUpdateReq_ref
 *  @author Asbjorn Steira Mikkelsen
 */

#include "epp_hostdata.h"
#include "epp_GenericReq.h"
#include "epp_HostUpdateAddRemove.h"
#include "epp_HostUpdateChange.h"

namespace eppobject { namespace host {
   
   /// Class containing elements used in host update requests
   /// @see epp_GenericReq for additional information
   class epp_HostUpdateReq : public epp::epp_GenericReq {
      
    public:
      eppobject::epp::epp_string_ref m_name; /**< Fully qualified name of the host object to be updated */
      epp_HostUpdateAddRemove_ref m_add; /**< Attribute values to be added to the object */
      epp_HostUpdateAddRemove_ref m_remove; /**< Attribute values to be removed from the object */
      epp_HostUpdateChange_ref m_change; /**< Object attribute values to be changed */
      
      /// Null constructor
      epp_HostUpdateReq() {};
      
      /// Destructor
      virtual ~epp_HostUpdateReq() {}
      
      /// Constructor, using simple types when it can
      /// @note Cannot set simple types to NULL
      epp_HostUpdateReq(const eppobject::epp::epp_Command_ref & _m_cmd,
			const eppobject::epp::epp_string & _m_name,
			const epp_HostUpdateAddRemove_ref & _m_add,
			const epp_HostUpdateAddRemove_ref & _m_remove,
			const epp_HostUpdateChange_ref & _m_change)
	: epp::epp_GenericReq(_m_cmd)
	{
	   m_name.ref(new eppobject::epp::epp_string(_m_name));
	   m_add = _m_add;
	   m_remove = _m_remove;
	   m_change = _m_change;
	};
      
      /// Constructor, using only references
      epp_HostUpdateReq(const eppobject::epp::epp_Command_ref & _m_cmd,
			const eppobject::epp::epp_string_ref & _m_name,
			const epp_HostUpdateAddRemove_ref & _m_add,
			const epp_HostUpdateAddRemove_ref & _m_remove,
			const epp_HostUpdateChange_ref & _m_change)
	: epp::epp_GenericReq(_m_cmd)
	{
	   m_name = _m_name;
	   m_add = _m_add;
	   m_remove = _m_remove;
	   m_change = _m_change;
	};
   };

   /// Typedef for the ref to the epp_HostUpdateReq class
   typedef refcnt_ptr<epp_HostUpdateReq> epp_HostUpdateReq_ref;
   
}}

#endif
