/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

#ifndef __EPP_LOGINREQ_H
#define __EPP_LOGINREQ_H

/** @file epp_LoginReq.h
 *  @brief Definition of epp_LoginReq and epp_LoginReq_ref
 *  @author Asbjorn Steira Mikkelsen
 */

#include "epp_eppdata.h"
#include "epp_GenericReq.h"
#include "epp_Options.h"

namespace eppobject { namespace epp {
   
   /// Class containing elements used in login requests
   /// @see epp_GenericReq for additional information
   class epp_LoginReq : public epp_GenericReq {
      
    public:
      epp_string_ref m_client_id; /**< Client identifier assigned to the client by the server */
      epp_string_ref m_password; /**< Client plain text password */
      epp_string_ref m_new_password; /**< New client plain text password */
      epp_Options_ref m_options; /**< Client options for the command/session */
      epp_objuri_seq_ref m_services; /**< Namespace URIs representing the objects to be managed during the session */
      epp_exturi_seq_ref m_extensions; /**< Namespace URIs representing object extensions to be managed during the session */
      
      /// Null constructor
      epp_LoginReq() {};
      
      /// Destructor
      virtual ~epp_LoginReq() {}
      
      /// Constructor, using only references
      epp_LoginReq(const epp_Command_ref & _m_cmd,
                   const epp_string_ref & _m_client_id,
		   const epp_string_ref & _m_password,
		   const epp_string_ref & _m_new_password,
		   const epp_Options_ref & _m_options,
		   const epp_objuri_seq_ref & _m_services,
		   const epp_exturi_seq_ref & _m_extensions)
	: epp_GenericReq(_m_cmd)
	{
	   m_client_id = _m_client_id;
	   m_password = _m_password;
	   m_new_password = _m_new_password;
	   m_options = _m_options;
	   m_services = _m_services;
	   m_extensions = _m_extensions;
	};
   };

   /// Typedef for the ref to the epp_LoginReq class
   typedef refcnt_ptr<epp_LoginReq> epp_LoginReq_ref;
   
}}

#endif
