/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

#ifndef __EPP_OPTIONS_H
#define __EPP_OPTIONS_H

/** @file epp_Options.h
 *  @brief Definition of epp_Options and epp_Options_ref
 *  @author Asbjorn Steira Mikkelsen
 */

#include "epp_eppdata.h"

namespace eppobject { namespace epp {
   
   /// Class containing elements used to describe client options for a command/session
   class epp_Options {
      
    public:
      epp_string_ref m_version; /**< Protocol version to be used for the command or ongoing server session */
      epp_string_ref m_lang; /**< Text response language to be used for the command or ongoing server session commands (iso639) */
      
      /// Null constructor
      epp_Options() {};
      
      /// Destructor
      virtual ~epp_Options() {}
      
      /// Constructor, using simple types when it can
      /// @note Cannot set simple types to NULL
      epp_Options(const epp_string & _m_version,
		  const epp_string & _m_lang)
	{
	   m_version.ref(new epp_string(_m_version));
	   m_lang.ref(new epp_string(_m_lang));
	};
      
      /// Constructor, using only references
      epp_Options(const epp_string_ref & _m_version,
		  const epp_string_ref & _m_lang)
	{
	   m_version = _m_version;
	   m_lang = _m_lang;
	};
   };
   
   /// Typedef for the ref to the epp_Options class
   typedef refcnt_ptr<epp_Options> epp_Options_ref;
   
}}

#endif
