/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

#ifndef __EPP_SERVICEMENU_H
#define __EPP_SERVICEMENU_H

/** @file epp_ServiceMenu.h
 *  @brief Definition of epp_ServiceMenu and epp_ServiceMenu_ref
 *  @author Asbjorn Steira Mikkelsen
 */

#include "epp_eppdata.h"

namespace eppobject { namespace epp {
   
   /// Class describing the services supported by the server
   class epp_ServiceMenu {
      
    public:
      epp_string_seq_ref m_versions; /**< Protocol versions supported by the server */
      epp_string_seq_ref m_langs; /**< ID of languages supported by the server (iso639) */
      epp_objuri_seq_ref m_services; /**< Namespace URIs representing the objects that the server is capable of managing */
      epp_exturi_seq_ref m_extensions; /**< Namespace URIs representing object extensions supported by the server */
      
      /// Null constructor
      epp_ServiceMenu() {};
      
      /// Destructor
      virtual ~epp_ServiceMenu() {}
      
      /// Constructor, using only references
      epp_ServiceMenu(const epp_string_seq_ref & _m_versions,
		      const epp_string_seq_ref & _m_langs,
		      const epp_objuri_seq_ref & _m_services,
		      const epp_exturi_seq_ref & _m_extensions)
	{
	   m_versions = _m_versions;
	   m_langs = _m_langs;
	   m_services = _m_services;
	   m_extensions = _m_extensions;
	};
   };
   
   /// Typedef for the ref to the epp_ServiceMenu class
   typedef refcnt_ptr<epp_ServiceMenu> epp_ServiceMenu_ref;
   
}}

#endif
