/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

#ifndef __EPP_XMLEXCEPTION_H
#define __EPP_XMLEXCEPTION_H

/** @file epp_XMLException.h
 *  @brief Definition of epp_XMLException
 *  @author Asbjorn Steira Mikkelsen
 */

#include "epp_eppdata.h"
#include <exception>
#include <string>

namespace eppobject { namespace epp {

   /// Exception class containing information about an error occuring while parsing XML
   class epp_XMLException : public std::exception {
      
    public:
      epp_string_ref m_error_message; /**< Text describing the encountered error */

      /// Null constructor
      epp_XMLException() {};
      
      /// Destructor
      virtual ~epp_XMLException() throw() {};

      /// Constructor with error data
      epp_XMLException(const epp_string & data)
	{
	   m_error_message.ref(new epp_string(data));
	};
      
      /// Returns the error string
      /// @deprecated Access the data directly instead
      virtual epp_string getString() const
	{
	   if(m_error_message == NULL)
	     return "";
	   else
	     return *m_error_message;
	};
      
      /// Constructor with error data, file name, and line number
      epp_XMLException(const std::string & file, const int lineno, const epp_string & data)
	{
	   std::string error_message;
	   char buf[10];
	   sprintf(buf,"%d",lineno);
	   error_message = file;
	   error_message += ":";
	   error_message += buf;
	   error_message += ": ";
	   error_message += data;
	   m_error_message.ref(new epp_string(error_message));
	};
   };
   
}}

#endif
