/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

/* $Header: /cvsroot/epp-rtk/epp-rtk/c++/src/data/epp_contactXMLbase.cc,v 1.17 2003/07/18 20:21:19 tubadanm Exp $ */

/** @file epp_contactXMLbase.cc
 *  @brief Definition of XML helper-functions for contact objects
 *  @author Asbjorn Steira Mikkelsen
 *  @see epp_contactXMLbase.h
 */


#include <map>
#include <domtools/dom_wrapper.h>
#include "epp_contactXMLbase.h"
#include "epp_XMLException.h"

using namespace std;
using namespace domtools;
using namespace eppobject::contact;
using namespace eppobject::epp;


void eppobject::contact::setCommandSchemaAttributes(eppobject::epp::EPP_output & outputobject)
{
   outputobject.putAttribute("xmlns:contact","urn:ietf:params:xml:ns:contact-1.0");
   outputobject.putAttribute("xsi:schemaLocation","urn:ietf:params:xml:ns:contact-1.0 contact-1.0.xsd");
};


void eppobject::contact::addAddressElement(eppobject::epp::EPP_output & outputobject,
					   const epp_ContactNameAddress_ref & element,
					   const eppobject::epp::epp_string & tag, /* = address */
					   const eppobject::epp::epp_boolean & update /* = false */)
{
   if(element == NULL) {
      return;
   }

   // This one is used in both update and create, hence extra
   // checks if it is a create element...
   // <fixme>not in use right now...</fixme>
   
   map<epp_ContactPostalInfoType, string> postalInfoMap;
   postalInfoMap[INT] = "int";
   postalInfoMap[LOC] = "loc";

   outputobject.beginTag(tag);
   outputobject.putAttribute("type", postalInfoMap[*element->m_type]);
   
   if(element->m_name != NULL) {
      outputobject.putTag("contact:name", *element->m_name);
   } // if "m_name"
   
   if(element->m_org != NULL) {
      outputobject.putTag("contact:org", *element->m_org);
   } // if "m_org"
   
   if(element->m_address != NULL) {
      outputobject.beginTag("contact:addr");
      
      if(element->m_address->m_street1 != NULL) {
	 outputobject.putTag("contact:street", *element->m_address->m_street1);
      }
      
      if(element->m_address->m_street2 != NULL) {
	 outputobject.putTag("contact:street", *element->m_address->m_street2);
      }
      
      if(element->m_address->m_street3 != NULL) {
	 outputobject.putTag("contact:street", *element->m_address->m_street3);
      }
      
      if(element->m_address->m_city != NULL) {
	 outputobject.putTag("contact:city", *element->m_address->m_city);
      }
      
      if(element->m_address->m_state_province != NULL) {
	 outputobject.putTag("contact:sp", *element->m_address->m_state_province);
      }
      
      if(element->m_address->m_postal_code != NULL) {
	 outputobject.putTag("contact:pc", *element->m_address->m_postal_code);
      }
      
      if(element->m_address->m_country_code != NULL) {
	 outputobject.putTag("contact:cc", *element->m_address->m_country_code);
      }
      
      outputobject.endTag("contact:addr");
      
   } // if "m_address"
   
   outputobject.endTag(tag);
}
   

epp_ContactNameAddress eppobject::contact::parseAddress(const domtools::dom_ptr & adrNode)
{
   epp_ContactNameAddress adrStruct;
   
   string addr_type = adrNode->getAttribute("type");
   map<string, epp_ContactPostalInfoType> postal_info_type_map;
   postal_info_type_map["loc"] = LOC;
   postal_info_type_map["int"] = INT;
   adrStruct.m_type.ref(new epp_ContactPostalInfoType(postal_info_type_map[addr_type]));
   
   dom_ptr CNAChildNode = dom_ptr(adrNode->getFirstChildElement());
   
   string elemPrefix = adrNode->getPrefix();
   
   while(!CNAChildNode->isNull()) {
      string nodeName = CNAChildNode->getNodeName();
      
      if(nodeName == string(elemPrefix + ":name")) {
	 adrStruct.m_name.ref(new epp_string(CNAChildNode->getNodeData()));
//	 cout << "name: " << *adrStruct.m_name << endl;
      } // if "name"
      
      else if(nodeName == string(elemPrefix + ":org")) {
	 adrStruct.m_org.ref(new epp_string(CNAChildNode->getNodeData()));
//	 cout << "org: " << *adrStruct.m_org << endl;
      } // if "org"
      
      else if(nodeName == string(elemPrefix + ":addr")) {
	 adrStruct.m_address.ref(new epp_ContactAddress());
	 
	 dom_ptr addrChildNode = dom_ptr(CNAChildNode->getFirstChildElement());
	 
	 if(addrChildNode->isNull()) {
	    throw epp_XMLException(__FILE__, __LINE__, "Empty addr-element in address");
	 }
	 
	 while(!addrChildNode->isNull()) {
	    string addrNodeName = addrChildNode->getNodeName();
	    
	    if(addrNodeName == string(elemPrefix + ":street")) {
	       string street = addrChildNode->getNodeData();
	       
	       if(adrStruct.m_address->m_street1 == NULL) {
		  adrStruct.m_address->m_street1.ref(new epp_string(street));
	       } else if(adrStruct.m_address->m_street2 == NULL) {
		  adrStruct.m_address->m_street2.ref(new epp_string(street));
	       } else if(adrStruct.m_address->m_street3 == NULL) {
		  adrStruct.m_address->m_street3.ref(new epp_string(street));
	       } else {
		  throw epp_XMLException(__FILE__, __LINE__, "Too many street-elements in addr-element");
	       }
//	       cout << "street: " << street << endl;
	    } // if "street"
	    
	    else if(addrNodeName == string(elemPrefix + ":city")) {
	       adrStruct.m_address->m_city.ref(new epp_string(addrChildNode->getNodeData()));
//	       cout << "city: " << *adrStruct.m_address->m_city << endl;
	    } // if "city"
	    
	    else if(addrNodeName == string(elemPrefix + ":sp")) {
	       adrStruct.m_address->m_state_province.ref(new epp_string(addrChildNode->getNodeData()));
//	       cout << "sp: " << *adrStruct.m_address->m_state_province << endl;
	    } // if "sp"
	    
	    else if(addrNodeName == string(elemPrefix + ":pc")) {
	       adrStruct.m_address->m_postal_code.ref(new epp_string(addrChildNode->getNodeData()));
//	       cout << "pc: " << *adrStruct.m_address->m_postal_code << endl;
	    } // if "pc"
	    
	    else if(addrNodeName == string(elemPrefix + ":cc")) {
	       adrStruct.m_address->m_country_code.ref(new epp_string(addrChildNode->getNodeData()));
//	       cout << "cc: " << *adrStruct.m_address->m_country_code << endl;
	    } // if "cc"
	    
	    addrChildNode = dom_ptr(addrChildNode->getNextSiblingElement());
	 } // while "addrChildNode"
	 
      } // if "addr"
      
      CNAChildNode = dom_ptr(CNAChildNode->getNextSiblingElement());
   } // while "CNAChildNode"
   
   return adrStruct;
   
}


eppobject::epp::epp_string eppobject::contact::returnStatusType(const epp_ContactStatusType & statusType)
{
   map<epp_ContactStatusType, epp_string> contactStatusTypeMap;
   
   contactStatusTypeMap[CLIENT_DELETE_PROHIBITED] = "clientDeleteProhibited";
   contactStatusTypeMap[SERVER_DELETE_PROHIBITED] = "serverDeleteProhibited";
   contactStatusTypeMap[CLIENT_TRANSFER_PROHIBITED] = "clientTransferProhibited";
   contactStatusTypeMap[SERVER_TRANSFER_PROHIBITED] = "serverTransferProhibited";
   contactStatusTypeMap[CLIENT_UPDATE_PROHIBITED] = "clientUpdateProhibited";
   contactStatusTypeMap[SERVER_UPDATE_PROHIBITED] = "serverUpdateProhibited";
   contactStatusTypeMap[LINKED] = "linked";
   contactStatusTypeMap[OK] = "ok";
   contactStatusTypeMap[PENDING_CREATE] = "pendingCreate";
   contactStatusTypeMap[PENDING_DELETE] = "pendingDelete";
   contactStatusTypeMap[PENDING_TRANSFER] = "pendingTransfer";
   contactStatusTypeMap[PENDING_UPDATE] = "pendingUpdate";
   
   if(contactStatusTypeMap.find(statusType) == contactStatusTypeMap.end()) {
      throw epp_XMLException(__FILE__, __LINE__, "Unknown ContactStatusType");
   }
   
   return contactStatusTypeMap[statusType];
}


epp_ContactStatusType eppobject::contact::returnStatusEnumType(const eppobject::epp::epp_string & statusType)
{
   map<epp_string, epp_ContactStatusType> contactStatusTypeMap;
   
   contactStatusTypeMap["clientDeleteProhibited"] = CLIENT_DELETE_PROHIBITED;
   contactStatusTypeMap["serverDeleteProhibited"] = SERVER_DELETE_PROHIBITED;
   contactStatusTypeMap["clientTransferProhibited"] = CLIENT_TRANSFER_PROHIBITED;
   contactStatusTypeMap["serverTransferProhibited"] = SERVER_TRANSFER_PROHIBITED;
   contactStatusTypeMap["clientUpdateProhibited"] = CLIENT_UPDATE_PROHIBITED;
   contactStatusTypeMap["serverUpdateProhibited"] = SERVER_UPDATE_PROHIBITED;
   contactStatusTypeMap["linked"] = LINKED;
   contactStatusTypeMap["ok"] = OK;
   contactStatusTypeMap["pendingCreate"] = PENDING_CREATE;
   contactStatusTypeMap["pendingDelete"] = PENDING_DELETE;
   contactStatusTypeMap["pendingTransfer"] = PENDING_TRANSFER;
   contactStatusTypeMap["pendingUpdate"] = PENDING_UPDATE;
   
   if(contactStatusTypeMap.find(statusType) == contactStatusTypeMap.end()) {
      throw epp_XMLException(__FILE__, __LINE__, "Unknown ContactStatusType");
   }
   
   return contactStatusTypeMap[statusType];
}





/*********************************23**********************************
 *                                                                   *
 *           Only deprecated functions below this, please...         *
 *                                                                   *
 *********************************23**********************************/

//lint -e1773
// "Info 1773: Attempt to cast away const (or volatile)"
// As the functions below are deprecated, we are not concerned about
// these warnings.



// Warning: This function is deprecated, use the dom_ptr version instead!
epp_ContactNameAddress eppobject::contact::parseAddress(const DOM_Node & adrNode)
{
   epp_ContactNameAddress adrStruct;
   
   DOMString NS = adrNode.getNamespaceURI();
   
   DOM_Node nodeName = ((DOM_Element*)&adrNode)->getElementsByTagNameNS(NS, "name").item(0);
   if(nodeName.isNull())
     {
	throw epp_XMLException(__FILE__, __LINE__, "Missing name-element in address");
     }
   
   string name;
   getNodeData(nodeName, name);
   adrStruct.m_name.ref(new epp_string(name));
//   cout << "name: " << *adrStruct.m_name << endl;
   
   DOM_Node nodeOrg = ((DOM_Element*)&adrNode)->getElementsByTagNameNS(NS, "org").item(0);
   if(!nodeOrg.isNull())
     {
	string org;
	getNodeData(nodeOrg, org);
	adrStruct.m_org.ref(new epp_string(org));
//	cout << "org: " << *adrStruct.m_org << endl;
     }

   DOM_Node nodeAddr = ((DOM_Element*)&adrNode)->getElementsByTagNameNS(NS, "addr").item(0);
   if(nodeAddr.isNull())
     {
	throw epp_XMLException(__FILE__, __LINE__, "Missing addr-element in address");
     }
   
   adrStruct.m_address.ref(new epp_ContactAddress());

   DOM_NodeList elemNodes = nodeAddr.getChildNodes();
   
   if(elemNodes.getLength() == 0)
     {
	throw epp_XMLException(__FILE__, __LINE__, "Empty addr-element in address");
     }
   
   DOMString elemPrefix = nodeAddr.getPrefix();
   // <FIXME>This might not always work in all namespaces... (host:, obj:, osv)...</fixme>
   
   // <FIXME>Should we check that all mandatory fields present?</fixme>
   for(unsigned int i = 0; i < elemNodes.getLength(); i++)
     {
	DOM_Node elNode = elemNodes.item(i);
	
	if(elNode.getNodeName().equals(elemPrefix + ":street")) {
	   string street;
	   getNodeData(elNode, street);
	   
	   if(adrStruct.m_address->m_street1 == NULL)
	     adrStruct.m_address->m_street1.ref(new epp_string(street));
	   else if(adrStruct.m_address->m_street2 == NULL)
	     adrStruct.m_address->m_street2.ref(new epp_string(street));
	   else if(adrStruct.m_address->m_street3 == NULL)
	     adrStruct.m_address->m_street3.ref(new epp_string(street));
	   else
	     throw epp_XMLException(__FILE__, __LINE__, "Too many street-elements in addr-element");
	   
//	   cout << "street: " << street << endl;
	}
	else if(elNode.getNodeName().equals(elemPrefix + ":city")) {
	   string city;
	   getNodeData(elNode, city);
	   adrStruct.m_address->m_city.ref(new epp_string(city));
//	   cout << "city: " << *adrStruct.m_address->m_city << endl;
	}
	else if(elNode.getNodeName().equals(elemPrefix + ":sp")) {
	   string sp;
	   getNodeData(elNode, sp);
	   adrStruct.m_address->m_state_province.ref(new epp_string(sp));
//	   cout << "sp: " << *adrStruct.m_address->m_state_province << endl;
	}
	else if(elNode.getNodeName().equals(elemPrefix + ":pc")) {
	   string pc;
	   getNodeData(elNode, pc);
	   adrStruct.m_address->m_postal_code.ref(new epp_string(pc));
//	   cout << "pc: " << *adrStruct.m_address->m_postal_code << endl;
	}
	else if(elNode.getNodeName().equals(elemPrefix + ":cc")) {
	   string cc;
	   getNodeData(elNode, cc);
	   adrStruct.m_address->m_country_code.ref(new epp_string(cc));
//	   cout << "cc: " << *adrStruct.m_address->m_country_code << endl;
	}
	
     }   
   
   return adrStruct;
}

//lint +e1773
