/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

#ifndef __EPP_HOSTDATA_H
#define __EPP_HOSTDATA_H

/** @file epp_hostdata.h
 *  @brief Declaration of enums/typedefs used in host objects
 *  @author Asbjorn Steira Mikkelsen
 */

#include "epp_eppdata.h"
#include <vector>


namespace eppobject { namespace host {
   

   /******************************************
    *                                        *
    *              Enumerations              *
    *                                        *
    ******************************************/

   /// Status types allowed for a host object
   enum epp_HostStatusType {
      CLIENT_DELETE_PROHIBITED,
      CLIENT_UPDATE_PROHIBITED,
      LINKED,
      OK,
      PENDING_CREATE,
      PENDING_DELETE,
      PENDING_TRANSFER,
      PENDING_UPDATE,
      SERVER_DELETE_PROHIBITED,
      SERVER_UPDATE_PROHIBITED	
   };
   
   /// Typedef for the ref to the #epp_HostStatusType enum
   typedef refcnt_ptr<epp_HostStatusType> epp_HostStatusType_ref;

   /// Type of host address
   enum epp_HostAddressType {
      IPV4,
      IPV6
   };
   
   /// Typedef for the ref to the #epp_HostAddressType enum
   typedef refcnt_ptr<epp_HostAddressType> epp_HostAddressType_ref;
   
   
   /******************************************
    *                                        *
    *              Pure typedefs             *
    *                                        *
    ******************************************/
   
   class epp_HostStatus; // Forward declaration...
   
   /// Typedef for a sequence of #epp_HostStatus elements
   typedef std::vector<epp_HostStatus> epp_host_status_seq;
   
   /// Typedef for the ref to #epp_host_status_seq
   typedef refcnt_ptr<epp_host_status_seq> epp_host_status_seq_ref;
   
   class epp_HostAddress; //Forward declaration...
   
   /// Typedef for a sequence of #epp_HostAddress elements
   typedef std::vector<epp_HostAddress> epp_host_address_seq;
   
   /// Typedef for the ref to #epp_host_address_seq
   typedef refcnt_ptr<epp_host_address_seq> epp_host_address_seq_ref;
   
}}

#endif
