/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

/* $Header: /cvsroot/epp-rtk/epp-rtk/c++/src/epp_ContactInfo.cc,v 1.19 2005/10/11 11:32:51 steira00 Exp $ */

#include "config.h"
#include "epp_ContactInfo.h"
#include <domtools/dom_output.h>
#include <domtools/dom_wrapper.h>
#include "data/epp_contactXMLbase.h"
#include "data/epp_XMLException.h"


using namespace std;
using namespace domtools;
using namespace eppobject::contact;
using namespace eppobject::epp;

void epp_ContactInfo::setRequestData(const epp_ContactInfoReq & parms)
{
   m_request.ref(new epp_ContactInfoReq(parms));
}

epp_ContactInfoRsp_ref epp_ContactInfo::getResponseData() const
{
   return m_response;
}


epp_string epp_ContactInfo::toXML()
{
   if(m_request == NULL || m_request->m_cmd == NULL || m_request->m_id == NULL) {
      throw epp_XMLException(__FILE__, __LINE__, "Missing data in request");
   }
   
   EPP_output xml_request;
   xml_request.setWhitespace(Whitespace);
   
   createStandardXMLHeader(xml_request);
   
   xml_request.beginTag("command");
   
   xml_request.beginTag("info");
   xml_request.beginTag("contact:info");

   setCommandSchemaAttributes(xml_request);
   
   xml_request.putTag("contact:id",*m_request->m_id);
   
   if(m_request->m_auth_info != NULL) {
      addAuthInfoElement(xml_request, m_request->m_auth_info, "contact:authInfo");
   }
   
   xml_request.endTag("contact:info");   
   xml_request.endTag("info");
   
   addExtensionElements(xml_request, m_request->m_cmd->m_extentions);
   
   if(m_request->m_cmd->m_client_trid != NULL) {
      xml_request.putTag("clTRID",*m_request->m_cmd->m_client_trid);
   } // if "m_cmd->m_client_trid"
   
   xml_request.endTag("command");
   
   createStandardXMLFooter(xml_request);
   
   return xml_request.getString();  
}


void epp_ContactInfo::fromXML(const eppobject::epp::epp_string & xml)
{
   dom_ptr doc = createDOMDocument(xml);
   
   dom_ptr responseNode = dom_ptr(doc->getTag("response"));
   
   if(responseNode->isNull()) {
      throw epp_XMLException(__FILE__, __LINE__, "Unable to find response-element");
   }
   
   m_response.ref(new epp_ContactInfoRsp());
   
   epp_Response_ref res = createGenericResponse(responseNode);
   m_response->m_rsp = res;
   
   dom_ptr infDataNode = dom_ptr(getresDataInfo(responseNode, "infData"));
   if(infDataNode->isNull()) {
      throw epp_XMLException(__FILE__, __LINE__, "Unable to find infData-element");
   }
   
   dom_ptr infChildNode = dom_ptr(infDataNode->getFirstChildElement());

   if(infChildNode->isNull()) {
      throw epp_XMLException(__FILE__, __LINE__, "Empty infData-element");
   }
   
   string elemPrefix = infDataNode->getPrefix();
   
   while(!infChildNode->isNull()) {
      
      string nodeName = infChildNode->getNodeName();
      
      if(nodeName == string(elemPrefix + ":roid")) {
	 m_response->m_roid.ref(new epp_roid(infChildNode->getNodeData()));
//	 cout << "roid: " << *m_response->m_roid << endl;
      } // if "roid"
      
      else if(nodeName == string(elemPrefix + ":id")) {
	 m_response->m_id.ref(new epp_string(infChildNode->getNodeData()));
//	 cout << "id: " << *m_response->m_id << endl;
      } // if "id"
      
      else if(nodeName == string(elemPrefix + ":status")) {
	 
	 if(m_response->m_status == NULL) {
	    m_response->m_status.ref(new epp_contact_status_seq);
	 }
	 
	 epp_ContactStatus cStatus;
	 
	 string type = infChildNode->getAttribute("s");
	 if(!type.empty()) {
	    cStatus.m_type.ref(new epp_ContactStatusType(returnStatusEnumType(type)));
	 }
	 
	 string lang = infChildNode->getAttribute("lang");
	 if(!lang.empty()) {
	    cStatus.m_lang.ref(new epp_string(lang));
	 }
	 
	 string value = infChildNode->getNodeData();
	 if(!value.empty()) {
	    cStatus.m_value.ref(new epp_string(value));
	 }
	 
	 m_response->m_status->push_back(cStatus);
//	 cout << "status: " << type << ", " << lang << ", " << value <<  endl;
      } // if "status"
      
      else if(nodeName == string(elemPrefix + ":postalInfo")) {
         if ( m_response->m_addresses == NULL ) {
            m_response->m_addresses.ref(new epp_ContactNameAddress_seq());
         }
         epp_ContactNameAddress cna(parseAddress(infChildNode));
	 m_response->m_addresses->push_back(cna);
         if ( cna.m_type != NULL ) {
            if ( *cna.m_type == LOC ) {
               m_response->m_i15d_address.ref(new epp_ContactNameAddress(cna));
            } else {
               m_response->m_ascii_address.ref(new epp_ContactNameAddress(cna));
            }
         }
      } // if "postalInfo"
	
      else if(nodeName == string(elemPrefix + ":voice")) {
	 m_response->m_voice.ref(new epp_ContactPhone());
	 
	 string extension = infChildNode->getAttribute("x");
	 if(!extension.empty())
	   m_response->m_voice->m_extension.ref(new epp_string(extension));
	 
	 string value = infChildNode->getNodeData();
	 if(!value.empty())
	   m_response->m_voice->m_value.ref(new epp_string(value));
	 
//	 cout << "voice: " << extension << ", " << value <<  endl;	     
      } // if "voice"
      
      else if(nodeName == string(elemPrefix + ":fax")) {
	 m_response->m_fax.ref(new epp_ContactPhone());
	 
	 string extension = infChildNode->getAttribute("x");
	 if(!extension.empty())
	   m_response->m_fax->m_extension.ref(new epp_string(extension));
	 
	 string value = infChildNode->getNodeData();
	 if(!value.empty())
	   m_response->m_fax->m_value.ref(new epp_string(value));
	 
//	 cout << "fax: " << extension << ", " << value <<  endl;
      } // if "fax"

      else if(nodeName == string(elemPrefix + ":email")) {
	 m_response->m_email.ref(new epp_string(infChildNode->getNodeData()));
//	 cout << "email: " << *m_response->m_email << endl;
      } // if "email"
      
      else if(nodeName == string(elemPrefix + ":clID")) {
	 m_response->m_client_id.ref(new epp_string(infChildNode->getNodeData()));
//	 cout << "clID: " << *m_response->m_client_id << endl;
      } // if "clID"
      
      else if(nodeName == string(elemPrefix + ":crID")) {
	 m_response->m_created_by.ref(new epp_string(infChildNode->getNodeData()));
//	 cout << "crID: " << *m_response->m_created_by << endl;
      } // if "crID"
      
      else if(nodeName == string(elemPrefix + ":crDate")) {
	 m_response->m_created_date.ref(new epp_datetime(infChildNode->getNodeData()));
//	 cout << "crDate: " << *m_response->m_created_date << endl;
      } // if "crDate"
      
      else if(nodeName == string(elemPrefix + ":upID")) {
	 m_response->m_updated_by.ref(new epp_string(infChildNode->getNodeData()));
//	 cout << "upID: " << *m_response->m_updated_by << endl;
      } // if "upID"
      
      else if(nodeName == string(elemPrefix + ":upDate")) {
	 m_response->m_updated_date.ref(new epp_datetime(infChildNode->getNodeData()));
//	 cout << "upDate: " << *m_response->m_updated_date << endl;
      } // if "upDate"
      
      else if(nodeName == string(elemPrefix + ":trDate")) {
	 m_response->m_transfer_date.ref(new epp_datetime(infChildNode->getNodeData()));
//	 cout << "trDate: " << *m_response->m_transfer_date << endl;
      } // if "trDate"
      
      else if(nodeName == string(elemPrefix + ":authInfo")) {
	 m_response->m_auth_info = createAuthInfoObject(infChildNode);
      } // if "authInfo"
      
      
      infChildNode = dom_ptr(infChildNode->getNextSiblingElement());
   } // while "infChildNode"
   
}
