/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

/* $Header: /cvsroot/epp-rtk/epp-rtk/c++/src/epp_ContactUpdate.cc,v 1.15 2003/07/18 20:21:19 tubadanm Exp $ */

#include "config.h"
#include "epp_ContactUpdate.h"
#include <domtools/dom_output.h>
#include <domtools/dom_wrapper.h>
#include "data/epp_contactXMLbase.h"
#include "data/epp_XMLException.h"


using namespace std;
using namespace domtools;
using namespace eppobject::contact;
using namespace eppobject::epp;


void epp_ContactUpdate::setRequestData(const epp_ContactUpdateReq & parms)
{
   m_request.ref(new epp_ContactUpdateReq(parms));
}

epp_ContactUpdateRsp_ref epp_ContactUpdate::getResponseData() const
{
   return m_response;
}


epp_string epp_ContactUpdate::toXML()
{
   if(m_request == NULL || m_request->m_cmd == NULL || m_request->m_id == NULL) {
      throw epp_XMLException(__FILE__, __LINE__, "Missing data in request");
   }
   
   EPP_output xml_request;
   xml_request.setWhitespace(Whitespace);
   
   createStandardXMLHeader(xml_request);
   
   xml_request.beginTag("command");
   
   xml_request.beginTag("update");
   xml_request.beginTag("contact:update");
   
   setCommandSchemaAttributes(xml_request);
   
   xml_request.putTag("contact:id",*m_request->m_id);
   
   addRemAddElement(xml_request, m_request->m_add, "add");
   addRemAddElement(xml_request, m_request->m_remove, "rem");
   addChangeElement(xml_request, m_request->m_change);   
   
   xml_request.endTag("contact:update");
   xml_request.endTag("update");
   
   addExtensionElements(xml_request, m_request->m_cmd->m_extentions);
   
   if(m_request->m_cmd->m_client_trid != NULL) {
      xml_request.putTag("clTRID",*m_request->m_cmd->m_client_trid);
   } // if "m_cmd->m_client_trid"
   
   xml_request.endTag("command");
   
   createStandardXMLFooter(xml_request);
   
   return xml_request.getString();  
}

void epp_ContactUpdate::fromXML(const eppobject::epp::epp_string & xml)
{
   dom_ptr doc = createDOMDocument(xml);
   
   dom_ptr responseNode = dom_ptr(doc->getTag("response"));
   
   if(responseNode->isNull()) {
      throw epp_XMLException(__FILE__, __LINE__, "Unable to find response-element");
   }
   
   epp_Response_ref res = createGenericResponse(responseNode);
   
   m_response.ref(new epp_ContactUpdateRsp(res));
}

void epp_ContactUpdate::addRemAddElement(eppobject::epp::EPP_output & outputobject,
					 const epp_ContactUpdateAddRemove_ref & element,
					 const eppobject::epp::epp_string & mode) const
{
   if(element == NULL) {
      return;
   }
   
   outputobject.beginTag("contact:" + mode);
   
   if(element->m_status != NULL) {
      
      for(epp_contact_status_seq::iterator i = element->m_status->begin();
	  i != element->m_status->end(); i++)
	{
	   if(i->m_type == NULL) {
	      throw epp_XMLException(__FILE__, __LINE__, "Missing data in contactstatus-list");
	   }
	   
	   outputobject.beginTag("contact:status");
	   outputobject.putAttribute("s",returnStatusType(*i->m_type));
	   
	   if(i->m_lang != NULL) {
	      outputobject.putAttribute("lang",*i->m_lang);
	   }
	   if(i->m_value != NULL) {
	      outputobject.putCDATA(*i->m_value);
	   }
	   
	   outputobject.endTag("contact:status");
	} // for
      
      outputobject.endTag("contact:" + mode);
   } // if "m_status"
}

void epp_ContactUpdate::addChangeElement(eppobject::epp::EPP_output & outputobject,
					 const epp_ContactUpdateChange_ref & element) const 
{
   if(element == NULL) {
      return;
   }
   
   outputobject.beginTag("contact:chg");
   
   if( element->m_ascii_address != NULL ||
       element->m_i15d_address  != NULL ) {
      addAddressElement(outputobject, element->m_ascii_address, "contact:postalInfo", true);
      addAddressElement(outputobject, element->m_i15d_address, "contact:postalInfo", true);
   } else if ( element->m_addresses != NULL &&
               element->m_addresses->size() > 0 ) {
      for(epp_ContactNameAddress_seq::iterator i = element->m_addresses->begin();
          i != element->m_addresses->end(); i++)
        {
         epp_ContactNameAddress_ref cnar;
         cnar.ref(new epp_ContactNameAddress(*i));
         addAddressElement(outputobject, cnar, "contact:postalInfo", true);
        }
      
   }
   
   if(element->m_voice != NULL) {
      
      outputobject.beginTag("contact:voice");
      
      if(element->m_voice->m_extension != NULL) {
	 outputobject.putAttribute("x", *element->m_voice->m_extension);
      }
      
      if(element->m_voice->m_value != NULL) {
	 outputobject.putCDATA(*element->m_voice->m_value);
      }
      outputobject.endTag("contact:voice");
   } // if "m_voice"
   
   if(element->m_fax != NULL) {
      
      outputobject.beginTag("contact:fax");
      if(element->m_fax->m_extension != NULL) {
	 outputobject.putAttribute("x", *element->m_fax->m_extension);
      }
      
      if(element->m_fax->m_value != NULL) {
	 outputobject.putCDATA(*element->m_fax->m_value);
      }
      
      outputobject.endTag("contact:fax");
   } // if "m_fax"
   
   if(element->m_email != NULL) {
      outputobject.putTag("contact:email", *element->m_email);
   } // if "m_email"
   
   if(element->m_auth_info != NULL) {
      addAuthInfoElement(outputobject, element->m_auth_info, "contact:authInfo");
   } // if "m_auth_info"

   outputobject.endTag("contact:chg");
}
