/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

#include "config.h"
#include "epp_DomainInfo.h"
#include <domtools/dom_output.h>
#include <domtools/dom_wrapper.h>
#include "data/epp_domainXMLbase.h"
#include "data/epp_XMLException.h"


using namespace std;
using namespace domtools;
using namespace eppobject::domain;
using namespace eppobject::epp;

void epp_DomainInfo::setRequestData(const epp_DomainInfoReq & parms)
{
   m_request.ref(new epp_DomainInfoReq(parms));
}

epp_DomainInfoRsp_ref epp_DomainInfo::getResponseData() const
{
   return m_response;
}


epp_string epp_DomainInfo::toXML()
{
   if(m_request == NULL || m_request->m_cmd == NULL || m_request->m_name == NULL) {
      throw epp_XMLException(__FILE__, __LINE__, "Missing data in request");
   }
   
   EPP_output xml_request;
   xml_request.setWhitespace(Whitespace);
   
   createStandardXMLHeader(xml_request);
   
   xml_request.beginTag("command");
   
   xml_request.beginTag("info");
   xml_request.beginTag("domain:info");
   
   setCommandSchemaAttributes(xml_request);
   
   xml_request.beginTag("domain:name");
   
   if(m_request->m_hosts_type != NULL) {
      xml_request.putAttribute("hosts", returnDomainHostsType(*m_request->m_hosts_type));
   } // if "m_hosts_type"
   
   xml_request.putCDATA(*m_request->m_name);
   
   xml_request.endTag("domain:name");
   
   if(m_request->m_auth_info != NULL) {
	   addAuthInfoElement(xml_request, m_request->m_auth_info, "domain:authInfo");
   } //

   xml_request.endTag("domain:info");   
   xml_request.endTag("info");
   
   addExtensionElements(xml_request, m_request->m_cmd->m_extentions);
   
   if(m_request->m_cmd->m_client_trid != NULL) {
      xml_request.putTag("clTRID",*m_request->m_cmd->m_client_trid);
   } // if "m_cmd->m_client_trid"
   
   xml_request.endTag("command");
   
   createStandardXMLFooter(xml_request);
   
   return xml_request.getString();  
}


void epp_DomainInfo::fromXML(const eppobject::epp::epp_string & xml)
{
   dom_ptr doc = createDOMDocument(xml);
   
   dom_ptr responseNode = dom_ptr(doc->getTag("response"));
   
   if(responseNode->isNull()) {
      throw epp_XMLException(__FILE__, __LINE__, "Unable to find response-element");
   }
   
   m_response.ref(new epp_DomainInfoRsp());
   
   epp_Response_ref res = createGenericResponse(responseNode);
   m_response->m_rsp = res;
   
   dom_ptr infDataNode = dom_ptr(getresDataInfo(responseNode, "infData"));
   
   if(infDataNode->isNull()) {
      throw epp_XMLException(__FILE__, __LINE__, "Unable to find infData-element");
   }
   
   dom_ptr infChildNode = dom_ptr(infDataNode->getFirstChildElement());
   
   if(infChildNode->isNull()) {
      throw epp_XMLException(__FILE__, __LINE__, "Empty infData-element");
   }
   
   string elemPrefix = infDataNode->getPrefix();
   
   while(!infChildNode->isNull()) {
      
      string nodeName = infChildNode->getNodeName();
      
      if(nodeName == string(elemPrefix + ":name")) {
	 m_response->m_name.ref(new epp_string(infChildNode->getNodeData()));
//	 cout << "name: " << *m_response->m_name << endl;
      } // if "name"
      
      else if(nodeName == string(elemPrefix + ":roid")) {
	 m_response->m_roid.ref(new epp_roid(infChildNode->getNodeData()));
//	 cout << "roid: " << *m_response->m_roid << endl;
      } // if "roid"
      
      else if(nodeName == string(elemPrefix + ":status")) {
	 
	 if(m_response->m_status == NULL) {
	    m_response->m_status.ref(new epp_domain_status_seq());
	 }
	 
	 epp_DomainStatus dStatus;
	 
	 string type = infChildNode->getAttribute("s");
	 if(!type.empty()) {
	    dStatus.m_type.ref(new epp_DomainStatusType(returnStatusEnumType(type)));
	 }
	 
	 string lang = infChildNode->getAttribute("lang");
	 if(!lang.empty()) {
	    dStatus.m_lang.ref(new epp_string(lang));
	 }
	 
	 string value = infChildNode->getNodeData();
	 if(!value.empty()) {
	    dStatus.m_value.ref(new epp_string(value));
	 }
	 
	 m_response->m_status->push_back(dStatus);
//	 cout << "status: " << type << ", " << lang << ", " << value <<  endl;
      } // if "status"
      
      else if(nodeName == string(elemPrefix + ":registrant")) {
	 m_response->m_registrant.ref(new epp_string(infChildNode->getNodeData()));
//	 cout << "registrant: " << *m_response->m_registrant << endl;
      } // if "registrant"
      
      else if(nodeName == string(elemPrefix + ":contact")) {
	 
	 if(m_response->m_contacts == NULL) {
	    m_response->m_contacts.ref(new epp_domain_contact_seq());
	 }
	 
	 epp_DomainContact dContact;
	 
	 string type = infChildNode->getAttribute("type");
	 if(!type.empty()) {
	    dContact.m_type.ref(new epp_DomainContactType(returnContactEnumType(type)));
	 }
	 
	 string id =  infChildNode->getNodeData();
	 if(!id.empty()) {
	    dContact.m_id.ref(new epp_string(id));
	 }
	 
	 m_response->m_contacts->push_back(dContact);
//	 cout << "contact: " << type << ", " << id <<  endl;
      } // if "contact"
      
      else if(nodeName == string(elemPrefix + ":ns")) {

          dom_ptr hostObjNode = dom_ptr(infChildNode->getFirstChildElement());

          if(hostObjNode->isNull()) {
              throw epp_XMLException(__FILE__, __LINE__, "Empty hostObj-element");
          }

          while(!hostObjNode->isNull()) {
              if (hostObjNode->getNodeName() != string(elemPrefix + ":hostObj")) {
                  throw epp_XMLException(__FILE__, __LINE__, string(hostObjNode->getNodeName() + " in domain:ns not supported"));
              }

              if(m_response->m_name_servers == NULL) {
                  m_response->m_name_servers.ref(new epp_string_seq());
              }

              m_response->m_name_servers->push_back(hostObjNode->getNodeData());
              hostObjNode = dom_ptr(hostObjNode->getNextSiblingElement());
          }
//	 cout << "ns: " << ns <<  endl;
      } // if "ns"
      
      else if(nodeName == string(elemPrefix + ":host")) {
	 
	 if(m_response->m_hosts == NULL) {
	    m_response->m_hosts.ref(new epp_string_seq());
	 }
	 
	 m_response->m_hosts->push_back(infChildNode->getNodeData());
//	 cout << "host: " << host <<  endl;	     
      } // if "host"
      
      else if(nodeName == string(elemPrefix + ":clID")) {
	 m_response->m_client_id.ref(new epp_string(infChildNode->getNodeData()));
//	 cout << "clID: " << *m_response->m_client_id << endl;
      } // if "clID"
      
      else if(nodeName == string(elemPrefix + ":crID")) {
	 m_response->m_created_by.ref(new epp_string(infChildNode->getNodeData()));
//	 cout << "crID: " << *m_response->m_created_by << endl;
      } // if "crID"
      
      else if(nodeName == string(elemPrefix + ":crDate")) {
	 m_response->m_created_date.ref(new epp_datetime(infChildNode->getNodeData()));
//	 cout << "crDate: " << *m_response->m_created_date << endl;
      } // if "crDate"
      
      else if(nodeName == string(elemPrefix + ":upID")) {
	 m_response->m_updated_by.ref(new epp_string(infChildNode->getNodeData()));
//	 cout << "upID: " << *m_response->m_updated_by << endl;
      } // if "upID"
      
      else if(nodeName == string(elemPrefix + ":upDate")) {
	 m_response->m_updated_date.ref(new epp_datetime(infChildNode->getNodeData()));
//	 cout << "upDate: " << *m_response->m_updated_date << endl;
      } // if "upDate"
      
      else if(nodeName == string(elemPrefix + ":exDate")) {
	 m_response->m_expiration_date.ref(new epp_datetime(infChildNode->getNodeData()));
//	 cout << "exDate: " << *m_response->m_expiration_date << endl;
      } // if "exDate"
      
      else if(nodeName == string(elemPrefix + ":trDate")) {
	 m_response->m_transfer_date.ref(new epp_datetime(infChildNode->getNodeData()));
//	 cout << "trDate: " << *m_response->m_transfer_date << endl;
      } // if "trDate"
      
      else if(nodeName == string(elemPrefix + ":authInfo")) {
	 m_response->m_auth_info = createAuthInfoObject(infChildNode);
      } // if "authInfo"
      
      infChildNode = dom_ptr(infChildNode->getNextSiblingElement());
   } // while "infChildNode"
   
}
