/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

#include "config.h"
#include "ltoa.h"
#include "epp_DomainTransfer.h"
#include <domtools/dom_output.h>
#include <domtools/dom_wrapper.h>
#include "data/epp_domainXMLbase.h"
#include "data/epp_XMLException.h"


using namespace std;
using namespace domtools;
using namespace eppobject::domain;
using namespace eppobject::epp;


void epp_DomainTransfer::setRequestData(const epp_DomainTransferReq & parms)
{
   m_request.ref(new epp_DomainTransferReq(parms));
}

epp_DomainTransferRsp_ref epp_DomainTransfer::getResponseData() const
{
   return m_response;
}


epp_string epp_DomainTransfer::toXML()
{
   if(m_request == NULL || m_request->m_cmd == NULL
      || m_request->m_name == NULL || m_request->m_trans == NULL)
     {
	throw epp_XMLException(__FILE__, __LINE__, "Missing data in request");
     }
   
   EPP_output xml_request;
   xml_request.setWhitespace(Whitespace);
   
   createStandardXMLHeader(xml_request);
   
   xml_request.beginTag("command");
   
   xml_request.beginTag("transfer");
   
   if(m_request->m_trans->m_op != NULL) {
      xml_request.putAttribute("op", returnTransferType(*m_request->m_trans->m_op));
   } // if "m_trans->m_op"
   
   xml_request.beginTag("domain:transfer");

   setCommandSchemaAttributes(xml_request);
   
   xml_request.putTag("domain:name",*m_request->m_name);
   
   if(m_request->m_period != NULL) {
      
      xml_request.beginTag("domain:period");
      if(m_request->m_period->m_unit != NULL) {
	 xml_request.putAttribute("unit", returnPeriodUnitType(*m_request->m_period->m_unit));
      }
      if(m_request->m_period->m_value != NULL) {
	 xml_request.putCDATA(ltoa(*m_request->m_period->m_value));
      }
      xml_request.endTag("domain:period");
   } // if "m_period"
   
   addAuthInfoElement(xml_request, m_request->m_trans->m_auth_info, "domain:authInfo");
      
   xml_request.endTag("domain:transfer");   
   xml_request.endTag("transfer");
   
   addExtensionElements(xml_request, m_request->m_cmd->m_extentions);
   
   if(m_request->m_cmd->m_client_trid != NULL) {
      xml_request.putTag("clTRID",*m_request->m_cmd->m_client_trid);
   } // if "m_cmd->m_client_trid"
   
   xml_request.endTag("command");
   
   createStandardXMLFooter(xml_request);
   
   return xml_request.getString();  
}

void epp_DomainTransfer::fromXML(const eppobject::epp::epp_string & xml)
{
   dom_ptr doc = createDOMDocument(xml);
   
   dom_ptr responseNode = dom_ptr(doc->getTag("response"));
   
   if(responseNode->isNull()) {
      throw epp_XMLException(__FILE__, __LINE__, "Unable to find response-element");
   }
   
   m_response.ref(new epp_DomainTransferRsp());
   
   epp_Response_ref res = createGenericResponse(responseNode);
   m_response->m_rsp = res;
   
   dom_ptr trnDataNode = dom_ptr(getresDataInfo(responseNode, "trnData"));
   
   if(trnDataNode->isNull()) {
      throw epp_XMLException(__FILE__, __LINE__, "Unable to find trnData-element");
   }
   
   dom_ptr trnChildNode = dom_ptr(trnDataNode->getFirstChildElement());
   
   if(trnChildNode->isNull()) {
      throw epp_XMLException(__FILE__, __LINE__, "Empty trnData-element");
   }
   
   m_response->m_trn_data.ref(new epp_DomainTrnData());
   
   string elemPrefix = trnDataNode->getPrefix();
   
   while(!trnChildNode->isNull()) {
      
      string nodeName = trnChildNode->getNodeName();
      
      if(nodeName == string(elemPrefix + ":name")) {
	 m_response->m_trn_data->m_name.ref(new epp_string(trnChildNode->getNodeData()));
//	 cout << "name: " << *m_response->m_trn_data->m_name << endl;
      } // if "name"
      
      else if(nodeName == string(elemPrefix + ":trStatus")) {
	 string trStatus = trnChildNode->getNodeData();
	 m_response->m_trn_data->m_transfer_status.ref(new epp_TransferStatusType(returnTransferStatusType(trStatus)));
//	 cout << "trStatus: " << trStatus << endl;
      } // if "trStatus"
      
      else if(nodeName == string(elemPrefix + ":reID")) {
	 m_response->m_trn_data->m_request_client_id.ref(new epp_string(trnChildNode->getNodeData()));
//	 cout << "reID: " << *m_response->m_trn_data->m_request_client_id << endl;
      } // if "reID"
      
      else if(nodeName == string(elemPrefix + ":reDate")) {
	 m_response->m_trn_data->m_request_date.ref(new epp_string(trnChildNode->getNodeData()));
//	 cout << "reDate: " << *m_response->m_trn_data->m_request_date << endl;
      } // if "reDate"
      
      else if(nodeName == string(elemPrefix + ":acID")) {
	 m_response->m_trn_data->m_action_client_id.ref(new epp_string(trnChildNode->getNodeData()));
//	 cout << "acID: " << *m_response->m_trn_data->m_action_client_id << endl;	     
      } // if "acID"
      
      else if(nodeName == string(elemPrefix + ":acDate")) {
	 m_response->m_trn_data->m_action_date.ref(new epp_string(trnChildNode->getNodeData()));
//	 cout << "acDate: " << *m_response->m_trn_data->m_action_date << endl;
      } // if "acDate"
      
      else if(nodeName == string(elemPrefix + ":exDate")) {
	 m_response->m_trn_data->m_expiration_date.ref(new epp_datetime(trnChildNode->getNodeData()));
//	 cout << "exDate: " << *m_response->m_trn_data->m_expiration_date << endl;
      } // if "exDate"
      
      trnChildNode = dom_ptr(trnChildNode->getNextSiblingElement());
   } // while "trnChildNode"
   
}
