/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

/* $Header: /cvsroot/epp-rtk/epp-rtk/c++/src/main_contactcreate.cc,v 1.19 2003/07/18 20:21:19 tubadanm Exp $ */

#include <vector>
#include <iostream>

#include "data/epp_ContactCreateReq.h"
#include "data/epp_ContactCreateRsp.h"
#include "data/epp_XMLException.h"
#include "data/epp_Exception.h"
#include "epp_Action.h"
#include "epp_ContactCreate.h"
#include "epp_Session.h"
#include "testTools.h"

using namespace std;
using namespace eppobject::contact;
using namespace eppobject::epp;

void printRsp(const epp_ContactCreateRsp_ref & req)
{
   cout << "Printing Response:\n_______________________________\n";
   
   if(req != NULL) {

      printResponsePart1(req->m_rsp);
      
      printResDataHeaders("contact", "creData");
      
      if(req->m_id != NULL)
	cerr << "<contact:id>" << *req-> m_id << "</contact:id>" << endl;
      
      if(req->m_creation_date != NULL)
	cerr << "<contact:crDate>" << *req-> m_creation_date << "</contact:crDate>" << endl;

      printResDataFooter("contact", "creData");
      printResponsePart2(req->m_rsp);
   }
   
   
   cout << "_______________________________\n";
} // printRsp


int main(int argc, char * argv[]) {
   
   epp_Session test;
   
   string username, password;
   
   string extra_usage("[-extradata=trid=xxx:contact=id/auth_info/name/org/street1/street2/street3/city/state/country/postal/phone/ext/email/fax/ext]\n");
   extra_usage.append("\tIf you included spaces in the contact name, org or address element,\n\tyou need to quote the entire 'contact' param\n");
   extra_usage.append("\tFor optional values, just leave them empty so that the slashes are back-to-back");
   if(!handleArgs(test, argc, argv, username, password, extra_usage))
     return 1;
   
   map<string,string> extraDataMap;
   
   if(argc > 1) {
      for(int parm = 1; parm < argc; parm++) {
	 
	 if (!strncmp(argv[parm], "-extradata=", 11)) {
#ifdef RTK_DEBUG
            cout << "Found extra data param..." << endl;
#endif
	    string td = &argv[parm][11];
	    extraDataMap = getExtraDataMap(td);
	 }
      }
   }
   
   testLogIn(test, username, password);
   
   try {
      
      epp_ContactCreateReq_ref l_req(new epp_ContactCreateReq());

      string trID = "ABC-12345";
      if(extraDataMap.find("trid") != extraDataMap.end()) trID = extraDataMap["trid"];
      
      l_req->m_cmd.ref(new epp_Command(NULL,
				       epp_trid(trID)));
            
      if( extraDataMap.find("contact") == extraDataMap.end() ) {
	 l_req->m_id.ref(new epp_string("sh8013"));

	 epp_ContactNameAddress_ref addr(new epp_ContactNameAddress());
	 addr->m_name.ref(new epp_string("John Doe"));
	 addr->m_org.ref(new epp_string("Example Inc."));
	 
	 addr->m_address.ref(new epp_ContactAddress());
	 addr->m_address->m_street1.ref(new epp_string("123 Example Dr."));
	 addr->m_address->m_street2.ref(new epp_string("Suite 100"));
	 addr->m_address->m_city.ref(new epp_string("Dulles"));
	 addr->m_address->m_state_province.ref(new epp_string("VA"));
	 addr->m_address->m_postal_code.ref(new epp_string("20166-6503"));
	 addr->m_address->m_country_code.ref(new epp_string("US"));

         addr->m_type.ref(new epp_ContactPostalInfoType(INT));
         l_req->m_addresses.ref(new epp_ContactNameAddress_seq());
	 l_req->m_addresses->push_back(*addr);
	 
	 l_req->m_voice.ref(new epp_ContactPhone(epp_string("1234"),
						 epp_string("+1.7035555555")));
	 l_req->m_fax.ref(new epp_ContactPhone());
	 l_req->m_fax->m_value.ref(new epp_string("+1.7035555556"));
	 
	 l_req->m_email.ref(new epp_string("jdoe@example.com"));
	 l_req->m_auth_info.ref(new epp_AuthInfo());
	 l_req->m_auth_info->m_type.ref(new epp_AuthInfoType(PW));
	 l_req->m_auth_info->m_value.ref(new epp_string("2fooBAR"));
	 
      } else {
      
         string contactDataStr = extraDataMap["contact"];
         vector<string> contactData;
         
         unsigned int end;
         while((end = contactDataStr.find('/')) != string::npos) {

	    string tmp = contactDataStr.substr(0,end);
#ifdef RTK_DEBUG
            cout << "The contact data string token ["<< tmp <<"]" << endl;
#endif
            contactData.push_back(tmp);

            contactDataStr.erase(0,end+1);

         } // while

         contactData.push_back(contactDataStr);
         while ( contactData.size() < 16 ) contactData.push_back("");


         cout << "building the contact...\n";
	 if(contactData[0] != "-1") l_req->m_id.ref(new epp_string(contactData[0]));
	 
	 if(contactData[1] != "-1") {
	    l_req->m_auth_info.ref(new epp_AuthInfo());
	    l_req->m_auth_info->m_type.ref(new epp_AuthInfoType(PW));
	    l_req->m_auth_info->m_value.ref(new epp_string(contactData[1]));
	 }
	 
	 epp_ContactNameAddress_ref addr(new epp_ContactNameAddress());
	 if(contactData[2] != "") addr->m_name.ref(new epp_string(contactData[2]));
	 if(contactData[3] != "") addr->m_org.ref(new epp_string(contactData[3]));
	 
	 addr->m_address.ref(new epp_ContactAddress());
	 if(contactData[4] != "-1") addr->m_address->m_street1.ref(new epp_string(contactData[4]));
	 if(contactData[5] != "-1") addr->m_address->m_street2.ref(new epp_string(contactData[5]));
	 if(contactData[6] != "-1") addr->m_address->m_street3.ref(new epp_string(contactData[6]));
	 if(contactData[7] != "-1") addr->m_address->m_city.ref(new epp_string(contactData[7]));
	 if(contactData[8] != "-1") addr->m_address->m_state_province.ref(new epp_string(contactData[8]));
	 if(contactData[9] != "-1") addr->m_address->m_country_code.ref(new epp_string(contactData[9]));
	 if(contactData[10] != "-1") addr->m_address->m_postal_code.ref(new epp_string(contactData[10]));
	 
         addr->m_type.ref(new epp_ContactPostalInfoType(INT));
         l_req->m_addresses.ref(new epp_ContactNameAddress_seq());
	 l_req->m_addresses->push_back(*addr);

	 if(contactData[11] != "-1" || contactData[12] != "-1") {
	    l_req->m_voice.ref(new epp_ContactPhone());
	    if(contactData[11] != "-1") l_req->m_voice->m_value.ref(new epp_string(contactData[11]));
	    if(contactData[12] != "-1") l_req->m_voice->m_extension.ref(new epp_string(contactData[12]));
	 }
	 
	 if(contactData[13] != "-1") l_req->m_email.ref(new epp_string(contactData[13]));

	 if(contactData[14] != "-1" || contactData[15] != "-1") {
	    l_req->m_fax.ref(new epp_ContactPhone());
	    if(contactData[14] != "-1") l_req->m_fax->m_value.ref(new epp_string(contactData[14]));
	    if(contactData[15] != "-1") l_req->m_fax->m_extension.ref(new epp_string(contactData[15]));
	 }

      }      
      
      cout << "Creating the contact create request...\n";
      
      epp_ContactCreate_ref l_obj(new epp_ContactCreate());
      
      l_obj->setRequestData(*l_req);
      
      test.processAction(l_obj);
      
      epp_ContactCreateRsp_ref l_rsp = l_obj->getResponseData();

#ifndef TEST_IO_REQUEST2CERR
      printRsp(l_rsp);
#endif 
      return 0;
      
   }
   catch(const epp_XMLException & ex) {
      cout << "Exception: " << ex.getString() << endl;
   }
   catch(const epp_Exception & ex) {
      cout << "Exception: " << endl;
#ifndef TEST_IO_REQUEST2CERR
      cerr << "<response>" << endl;
      printResultsSeq(ex.m_details);
      printTransID(ex.m_trans_id);
      cerr << "</response>" << endl;
#endif
   }
   catch(const epptransport::epp_TrException & ex) {
      cout << "Exception: " << ex.getString() << endl;
   }
   catch(...) {
      cout << "Something else went wrong...\n";
   }
   
}
