/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

/* $Header: /cvsroot/epp-rtk/epp-rtk/c++/src/main_contactupdate.cc,v 1.19 2003/07/18 20:21:19 tubadanm Exp $ */

#include <vector>
#include <iostream>

#include "data/epp_ContactUpdateReq.h"
#include "data/epp_ContactUpdateRsp.h"
#include "data/epp_XMLException.h"
#include "data/epp_Exception.h"
#include "data/epp_contactXMLbase.h"
#include "epp_Action.h"
#include "epp_ContactUpdate.h"
#include "epp_Session.h"
#include "testTools.h"

using namespace std;
using namespace eppobject::contact;
using namespace eppobject::epp;


void printRsp(const epp_ContactUpdateRsp_ref & req)
{
   cout << "Printing Response:\n_______________________________\n";
   
   if(req != NULL)
     printResponsePart(req->m_rsp);
   
   cout << "_______________________________\n";
} // printRsp


int main(int argc, char * argv[]) {
   
   epp_Session test;
   
   string username, password;
   
   vector<string> extra_usage;
   extra_usage.push_back("[-extradata=trid=xxx:id=abc");
   extra_usage.push_back(" -add=status1=aaa:status2=bbb/reason{:...}");
   extra_usage.push_back(" -remove=status1=ccc/reason:status2=ddd{:...}");
   extra_usage.push_back(" -change=name=Johnny:org=ACME:authinfo=pw/123123{:...}]");
   extra_usage.push_back("If you include spaces in the status reason, you need to quote the entire 'add' or 'remove' param");
   if(!handleArgs(test, argc, argv, username, password, extra_usage))
     return 1;
   
   map<string,string> extraDataMap;
   map<string,string> addDataMap;
   map<string,string> changeDataMap;
   map<string,string> removeDataMap;
   
   if(argc > 1) {
      for(int parm = 1; parm < argc; parm++) {
	 
	 if (!strncmp(argv[parm], "-extradata=", 11)) {
#ifdef RTK_DEBUG
            cout << "Found extra data param..." << endl;
#endif
	    string td = &argv[parm][11];
	    extraDataMap = getExtraDataMap(td);
	 }

	 if (!strncmp(argv[parm], "-add=", 5)) {
#ifdef RTK_DEBUG
            cout << "Found 'add' data param..." << endl;
#endif
	    string td = &argv[parm][5];
	    addDataMap = getExtraDataMap(td);
	 }

	 if (!strncmp(argv[parm], "-change=", 8)) {
#ifdef RTK_DEBUG
            cout << "Found 'change' data param..." << endl;
#endif
	    string td = &argv[parm][8];
	    changeDataMap = getExtraDataMap(td);
	 }

	 if (!strncmp(argv[parm], "-remove=", 8)) {
#ifdef RTK_DEBUG
            cout << "Found 'remove' data param..." << endl;
#endif
	    string td = &argv[parm][8];
	    removeDataMap = getExtraDataMap(td);
	 }
      }
   }
   
   testLogIn(test, username, password);
   
   try {
       
      epp_ContactUpdateReq_ref l_req(new epp_ContactUpdateReq());
   
      string trID = "ABC-12345";
      if(extraDataMap.find("trid") != extraDataMap.end()) trID = extraDataMap["trid"];
      
      l_req->m_cmd.ref(new epp_Command(NULL,
				       epp_trid(trID)));
      
      if ( extraDataMap.find("id") == extraDataMap.end() ) l_req->m_id.ref(new epp_string("sh8013"));
      else l_req->m_id.ref(new epp_string(extraDataMap["id"]));

      if( addDataMap.empty() &&
          removeDataMap.empty() &&
          changeDataMap.empty() ) {
	 epp_ContactNameAddress_ref addr(new epp_ContactNameAddress());
	 
	 addr->m_type.ref(new epp_ContactPostalInfoType(INT));
	 addr->m_org.ref(new epp_string(""));
	 addr->m_address.ref(new epp_ContactAddress());
	 addr->m_address->m_street1.ref(new epp_string("124 Example Dr."));
	 addr->m_address->m_street2.ref(new epp_string("Suite 200"));
	 addr->m_address->m_city.ref(new epp_string("Dulles"));
	 addr->m_address->m_state_province.ref(new epp_string("VA"));
	 addr->m_address->m_postal_code.ref(new epp_string("20166-6503"));
	 addr->m_address->m_country_code.ref(new epp_string("US"));
	 
	 epp_ContactUpdateChange_ref chg(new epp_ContactUpdateChange());
	 chg->m_ascii_address = addr;
	 chg->m_voice.ref(new epp_ContactPhone());
	 chg->m_voice->m_value.ref(new epp_string("+1.7034444444"));
	 
	 chg->m_fax.ref(new epp_ContactPhone());
	 chg->m_fax->m_value.ref(new epp_string(""));
	 chg->m_auth_info.ref(new epp_AuthInfo());
	 chg->m_auth_info->m_type.ref(new epp_AuthInfoType(PW));
	 chg->m_auth_info->m_value.ref(new epp_string("2BARfoo"));
	 
	 epp_ContactStatus status;
	 status.m_type.ref(new epp_ContactStatusType(CLIENT_DELETE_PROHIBITED));
         status.m_value.ref(new epp_string("don't ever leave me"));
	 
	 l_req->m_change = chg;
	 l_req->m_add.ref(new epp_ContactUpdateAddRemove());
	 l_req->m_add->m_status.ref(new epp_contact_status_seq());
	 l_req->m_add->m_status->push_back(status);
      }
      else {
	 if( ! addDataMap.empty() ) {
	    l_req->m_add.ref(new epp_ContactUpdateAddRemove());
	    
	    if( ! addDataMap["status1"].empty() ) {
	       char status_name[10];
               int status_number = 1;
               sprintf(status_name, "status%d", status_number);

	       l_req->m_add->m_status.ref(new epp_contact_status_seq());
	       
	       while( ! addDataMap[status_name].empty() ) {
		  string tmp = addDataMap[status_name];
		  
		  epp_ContactStatus status;

                  string type, value;
                  unsigned int sep = tmp.find("/");
                  if ( sep != string::npos ) {
                     type  = tmp.substr(0,sep);
                     value = tmp.substr(sep+1);
                     cout << "type ["<<type<<"] value ["<<value<<"]\n";
		     status.m_type.ref(new epp_ContactStatusType(returnStatusEnumType(type)));
	             status.m_value.ref(new epp_string(value));
                  } else {
		     status.m_type.ref(new epp_ContactStatusType(returnStatusEnumType(tmp)));
                  }
		  l_req->m_add->m_status->push_back(status);

                  status_number++;
                  sprintf(status_name, "status%d", status_number);
	       }
	    }
	 }
	 
	 if( ! removeDataMap.empty() ) {
	    l_req->m_remove.ref(new epp_ContactUpdateAddRemove());
	    
	    if( ! removeDataMap["status1"].empty() ) {
	       char status_name[10];
               int status_number = 1;
               sprintf(status_name, "status%d", status_number);

	       l_req->m_remove->m_status.ref(new epp_contact_status_seq());
	       
	       while( ! removeDataMap[status_name].empty() ) {
		  string tmp = removeDataMap[status_name];
		  
		  epp_ContactStatus status;

                  string type, value;
                  unsigned int sep = tmp.find("/");
                  if ( sep != string::npos ) {
                     type  = tmp.substr(0,sep);
                     value = tmp.substr(sep+1);
                     cout << "type ["<<type<<"] value ["<<value<<"]\n";
		     status.m_type.ref(new epp_ContactStatusType(returnStatusEnumType(type)));
	             status.m_value.ref(new epp_string(value));
                  } else {
		     status.m_type.ref(new epp_ContactStatusType(returnStatusEnumType(tmp)));
                  }
		  l_req->m_remove->m_status->push_back(status);
		  
                  status_number++;
                  sprintf(status_name, "status%d", status_number);
	       }
	    }
	 }
	 
	 if( ! changeDataMap.empty() ) {
	    
	    epp_ContactUpdateChange_ref chg(new epp_ContactUpdateChange());
	    
	    if(changeDataMap.find("authinfo") != changeDataMap.end()) {	    
               string authinfo = changeDataMap["authinfo"];
               string type, value;
               unsigned int sep = authinfo.find("/");
               if ( sep != string::npos ) {
                  type = authinfo.substr(0,sep);
                  value = authinfo.substr(sep+1);
	          l_req->m_change->m_auth_info.ref(new epp_AuthInfo());
                  if ( type == "pw" )  l_req->m_change->m_auth_info->m_type.ref(new epp_AuthInfoType(PW));
                  if ( type == "ext" ) l_req->m_change->m_auth_info->m_type.ref(new epp_AuthInfoType(EXT));
	          l_req->m_change->m_auth_info->m_value.ref(new epp_string(value));
               }
	    }

	    epp_ContactNameAddress_ref addr(new epp_ContactNameAddress());	    
	    addr->m_address.ref(new epp_ContactAddress());
	    
	    if(changeDataMap.find("name") != changeDataMap.end()) addr->m_name.ref(new epp_string(changeDataMap["name"]));
	    
	    if(changeDataMap.find("org") != changeDataMap.end()) addr->m_org.ref(new epp_string(changeDataMap["org"]));
	    
	    if(changeDataMap.find("street1") != changeDataMap.end()) addr->m_address->m_street1.ref(new epp_string(changeDataMap["street1"]));
	    
	    if(changeDataMap.find("street2") != changeDataMap.end()) addr->m_address->m_street2.ref(new epp_string(changeDataMap["street2"]));
	    
	    if(changeDataMap.find("street3") != changeDataMap.end()) addr->m_address->m_street3.ref(new epp_string(changeDataMap["street3"]));
	    
	    if(changeDataMap.find("city") != changeDataMap.end()) addr->m_address->m_city.ref(new epp_string(changeDataMap["city"]));
	    
	    if(changeDataMap.find("state") != changeDataMap.end()) addr->m_address->m_state_province.ref(new epp_string(changeDataMap["state"]));
	    
	    if(changeDataMap.find("country") != changeDataMap.end()) addr->m_address->m_country_code.ref(new epp_string(changeDataMap["country"]));
	    
	    if(changeDataMap.find("postal") != changeDataMap.end()) addr->m_address->m_postal_code.ref(new epp_string(changeDataMap["postal"]));
	    
	    if( changeDataMap.find("phone") != changeDataMap.end() ) {
	       chg->m_voice.ref(new epp_ContactPhone());
               string phone = changeDataMap["phone"];
               string extension = "";
               unsigned int x_location = phone.find("x");
               if ( x_location != string::npos ) {
                  extension = phone.substr(x_location+1);
                  phone = phone.substr(0,x_location);
	          chg->m_voice->m_extension.ref(new epp_string(extension));
               }
	       chg->m_voice->m_value.ref(new epp_string(phone));
	    }
	    
	    if(changeDataMap.find("email") != changeDataMap.end()) chg->m_email.ref(new epp_string(changeDataMap["email"]));
	    
	    if( changeDataMap.find("fax") != changeDataMap.end() ) {
	       chg->m_fax.ref(new epp_ContactPhone());
               string phone = changeDataMap["phone"];
               string extension = "";
               unsigned int x_location = phone.find("x");
               if ( x_location != string::npos ) {
                  extension = phone.substr(x_location+1);
                  phone = phone.substr(0,x_location);
	          chg->m_fax->m_extension.ref(new epp_string(extension));
               }
	       chg->m_fax->m_value.ref(new epp_string(phone));
	    }	    
	    
            chg->m_addresses.ref(new epp_ContactNameAddress_seq());
	    chg->m_addresses->push_back(*addr);
	    l_req->m_change = chg;
	 }
	 
      }
      
      epp_ContactUpdate_ref l_obj(new epp_ContactUpdate());
      
      l_obj->setRequestData(*l_req);
      
      test.processAction(l_obj);
      
      epp_ContactUpdateRsp_ref l_rsp = l_obj->getResponseData();
      
      
#ifndef TEST_IO_REQUEST2CERR
      printRsp(l_rsp);
#endif
      return 0;

   }
   catch(const epp_XMLException & ex) {
      cout << "Exception: " << ex.getString() << endl;
   }
   catch(const epp_Exception & ex) {
      cout << "Exception: " << endl;
#ifndef TEST_IO_REQUEST2CERR
      cerr << "<response>" << endl;
      printResultsSeq(ex.m_details);
      printTransID(ex.m_trans_id);
      cerr << "</response>" << endl;
#endif
   }
   catch(const epptransport::epp_TrException & ex) {
      cout << "Exception: " << ex.getString() << endl;
   }
   catch(...) {
      cout << "Something else went wrong...\n";
   }
   
}
