/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

/* $Header: /cvsroot/epp-rtk/epp-rtk/c++/src/main_loginunspec.cc,v 1.12 2003/07/18 20:21:19 tubadanm Exp $ */

#include <string>
#include <vector>
#include <iostream>

#include "data/epp_LoginReq.h"
#include "data/epp_LoginRsp.h"
#include "data/epp_XMLException.h"
#include "data/epp_Exception.h"
#include "epp_Action.h"
#include "epp_Login.h"
#include "epp_Session.h"
#include "testTools.h"
#include "unspec/epp_UnspecTest.h"

using namespace std;
using namespace eppobject::epp;

void printRsp(const epp_LoginRsp_ref & req)
{
   cout << "Printing Response:\n_______________________________\n";
   
   if(req != NULL)
     printResponsePart(req->m_rsp);

   
   cout << "_______________________________\n";
} // printRsp


int main(int argc, char * argv[]) {
   
   epp_Session test;
   
   string username, password;
   
   string extra_usage = "[-extradata=trid=xxx:newpw=yyy]";
   if(!handleArgs(test, argc, argv, username, password, extra_usage))
     return 1;
   
   map<string,string> extraDataMap;
   
   if(argc > 1) {
      for(int parm = 1; parm < argc; parm++) {
	 
	 if (!strncmp(argv[parm], "-extradata=", 11)) {
            cout << "Found extra data param..." << endl;
	    string td = &argv[parm][11];
	    extraDataMap = getExtraDataMap(td);
	 }
      }
   }
   
   try {

      epp_Command_ref cmd(new epp_Command());

      // Requesting weather forecast through Extension:
      epp_extension_seq_ref exts(new epp_extension_seq);
      epp_UnspecTest unspec_test;
      exts->push_back((epp_Extension)unspec_test);
      cmd->m_extentions = exts;
      string trID;
      if(extraDataMap.empty()) trID = "ABC-12345";
      else trID = extraDataMap["trid"];
      cmd->m_client_trid.ref(new epp_trid(trID));
      
      epp_objuri_seq_ref sers(new epp_objuri_seq);
      sers->push_back("urn:ietf:params:xml:ns:contact-1.0");
      sers->push_back("urn:ietf:params:xml:ns:domain-1.0");
      sers->push_back("urn:ietf:params:xml:ns:host-1.0");
      
      epp_exturi_seq_ref extSers(new epp_exturi_seq);
      extSers->push_back("http://custom/obj1ext-1.0");
      
      epp_LoginReq_ref l_req(new epp_LoginReq());
      l_req->m_cmd = cmd;
      l_req->m_services = sers;
      l_req->m_extensions = extSers;
      
      epp_Login_ref l_obj(new epp_Login());
      
      // Fill in credentials part of the login
      l_req->m_client_id.ref(new epp_string(username));
      l_req->m_password.ref(new epp_string(password));
      if(extraDataMap.find("newpw") != extraDataMap.end()) {
	 l_req->m_new_password.ref(new epp_string(extraDataMap["newpw"]));
      }
      // Fill in options part of request
      epp_Options_ref ops(new epp_Options(epp_string("1.0"),
					  epp_string("en")));
      l_req->m_options = ops;

      l_obj->setRequestData(*l_req);
	 
      test.connectAndGetGreeting();
      
      test.processAction(l_obj);
      
      epp_LoginRsp_ref l_rsp = l_obj->getResponseData();
      
#ifndef TEST_IO_REQUEST2CERR
	printRsp(l_rsp);
#endif 
      
      // Let us do something more with the extension:
      if(l_rsp->m_rsp->m_ext_strings != NULL)
	{
	   for(epp_string_seq::iterator i = l_rsp->m_rsp->m_ext_strings->begin();
	       i != l_rsp->m_rsp->m_ext_strings->end(); i++)
	     {
	        epp_UnspecTest_ref extension(new epp_UnspecTest());
	        extension->fromXML(epp_string(*i));
	        epp_Weather_ref weather = extension->getResponseData();
	        if(weather != NULL)
	          {
		     if(weather->m_today != NULL)
		       cout << "<today>" << *weather->m_today << "</today>" << endl;

		     if(weather->m_tomorrow != NULL)
		       cout << "<tomorrow>" << *weather->m_tomorrow << "</tomorrow>" << endl;
	          }
	     }
	}
      
      return 0;
      
   }
   catch(const epp_XMLException & ex) {
      cout << "Exception: " << ex.getString() << endl;
   }
   catch(const epp_Exception & ex) {
      cout << "Exception: " << endl;
#ifndef TEST_IO_REQUEST2CERR
      cerr << "<response>" << endl;
      printResultsSeq(ex.m_details);
      printTransID(ex.m_trans_id);
      cerr << "</response>" << endl;
#endif
   }
   catch(const epptransport::epp_TrException & ex) {
      cout << "Exception: " << ex.getString() << endl;
   }
   catch(...) {
      cout << "Something else went wrong...\n";
   }
}
