/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

#ifndef npssl_h_included
#define npssl_h_included


#include <openssl/ssl.h>
#include <openssl/err.h>
#include <openssl/x509.h>
#include <string>


namespace npbase {
   
   class npssl {
      SSL *ssl;
      SSL_CTX *ctx;
      int bufsiz;
      char *buf;
      bool server;
      
      std::string int_inbuffer;
      std::string int_outbuffer;
      std::string readbuffer;
      
      void ssl_error(void);
      
    public:
      
      void setVerifyLocations(const std::string &certname, const std::string &pathname);
      void setClientVerification(int mode,int (*verify_callback)(int, X509_STORE_CTX *));
      void setClientVerificationDepth(int depth);
      void newConnection(void);
      void setExData(int myIndex, void * data);
      X509 * getPeerCertificate( void );
      long getVerifyResult( void );
      
      void setCertFile(const std::string &filename);
      bool handShake(int infd, int outfd);
      
      int get_rfd(void);
      int get_wfd(void);
      
      // get functions
      std::string getS(void);
      char getch(void);
      // The pointer is only borrowed; the data must be copied.
      const char* getchars(long int);
      std::string bufferedGetS(void);
      char bufferedGetch(void);
      void bufferedUnGetch(char c);
      
      bool hasMore(void) const;
      
      // put functions
      bool putS(const std::string &str);
      bool putch(char c);
      bool putchars(const char *c, int);
      bool bufferedPutS(const std::string &str);
      bool bufferedPutch(char c);
      
      bool flush(void);
      
      npssl(int myBufsiz, bool mode);
      ~npssl();
   };

}


#endif
