/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/


/* $Header: /cvsroot/epp-rtk/epp-rtk/c++/src/testTools.cc,v 1.26 2003/07/18 20:21:19 tubadanm Exp $ */

/** @file testTools.cc
 *  @brief Definition of helper functions used for testing toolkit
 *  @author Asbjorn Steira Mikkelsen
 *  @see testTools.h
 */

#include <map>
#include "testTools.h"
#include "data/epp_Exception.h"
#include "data/epp_XMLException.h"
#include "data/epp_LoginReq.h"
#include "data/epp_LoginRsp.h"
#include "epp_Action.h"
#include "epp_Login.h"
#include "transport/transports.h"

using namespace std;
using namespace eppobject::epp;
using namespace epptransport;

void printResultsSeq(const epp_result_seq_ref & results)
{
   if(results != NULL) {
      
      for(epp_result_seq::iterator i = results->begin();
	  i != results->end(); i++)
	{
	   cerr << "<result";
	   
	   if(i->m_code != NULL) {
	      cerr << " code=\"" << *i->m_code << "\"";
	   } // if "m_code"
	   
	   cerr << ">" << endl;
	   
	   if(i->m_msg != NULL) {
	      
	      cerr << "<msg";
	      
	      if(i->m_lang != NULL) {
		 cerr << " lang=\"" << *i->m_lang << "\"";
	      }
	      
	      cerr << ">" << *i->m_msg << "</msg>" << endl;
	   } // if "m_msg"
	   
	   
	   if(i->m_values != NULL) {
	      
	      for(epp_resultvalue_seq::iterator j = i->m_values->begin();
		  j != i->m_values->end(); j++)
		{
                    // DANDAN
                    // XXX fix this to properly show the struct
                    cerr << "<value";
                    if ( j->m_namespace != NULL ) cerr << " xmlns=\"" << *j->m_namespace << "\"";
                    cerr << ">";
                    if ( j->m_element_name != NULL ) {
                        cerr << "<"<<*j->m_element_name<<">";
                        if ( j->m_element_value != NULL ) cerr << *j->m_element_value;
                        cerr << "</"<<*j->m_element_name<<">";
                    }
                    cerr << "</value>" << endl;
                    if ( j->m_xml_string != NULL ) cerr << "value xml: ["<<*j->m_xml_string<<"]" << endl;
		}
	   } // if "m_values"

	   cerr << "</result>" << endl;
	} // for
   } // if "results"
   
} // printResultsSeq


void printTransID(const eppobject::epp::epp_TransID_ref & transID)
{
   if(transID != NULL) {
      cerr << "<trID>" << endl;
      if(transID->m_client_trid != NULL) {
	 cerr << "<clTRID>" << *transID->m_client_trid << "</clTRID>" << endl;
      }
      if(transID->m_server_trid != NULL) {
	 cerr << "<svTRID>" << *transID->m_server_trid << "</svTRID>" << endl;
      }
      cerr << "</trID>" << endl;
   }
} // printTransID


void printResponsePart(const epp_Response_ref & response)
{
   printResponsePart1(response);
   printResponsePart2(response);
} // printResponsePart

void printResponsePart1(const epp_Response_ref & response)
{
   if(response != NULL) {
      
      cerr << "<response>" << endl;
      
      printResultsSeq(response->m_results);
      
      if(response->m_ext_strings != NULL) {
	 cerr << "<extension>" << endl;
         for(epp_xml_string_seq::iterator i = response->m_ext_strings->begin();
	     i != response->m_ext_strings->end(); i++)
	   {
	    cerr << *i << endl;
	   }
	 cerr << "</extension>" << endl;
      } // if "m_ext_string"
      
      if(response->m_message_queue != NULL) {
	 cerr << "<msgQ";
	 if(response->m_message_queue->m_count != NULL) {
	    cerr << " count=\"" << *response->m_message_queue->m_count << "\"";
	 }
	 if(response->m_message_queue->m_id != NULL) {
	    cerr << " id=\"" << *response->m_message_queue->m_id << "\"";
	 }
	 
	 if( response->m_message_queue->m_queue_date != NULL ||
             response->m_message_queue->m_msg != NULL ) {

	    cerr << ">" << endl;

	    if(response->m_message_queue->m_msg != NULL) {
	       cerr << "\t<msg";
	       if(response->m_message_queue->m_msg->m_lang != NULL) {
	          cerr << " lang=\"" << *response->m_message_queue->m_msg->m_lang << "\"";
	       }
               cerr << ">" << *response->m_message_queue->m_msg->m_value << "</msg>" << endl;
	    }
	    if(response->m_message_queue->m_queue_date != NULL) {
	       cerr << "\t<qDate>" << *response->m_message_queue->m_queue_date << "</qDate>" << endl;
	    }

	    cerr << "</msgQ>" << endl;
         }
	 else {
	    cerr << "/>" << endl;
	 }
	 
      } // if "m_message_queue"
      
   } // if "response"

} // printResponsePart1

void printResponsePart2(const epp_Response_ref & response)
{
   if(response != NULL) {
      
      if(response->m_trans_id != NULL) {
	 cerr << "<trID>\n";
	 if(response->m_trans_id->m_client_trid != NULL) {
	    cerr << "<clTRID>" << *response->m_trans_id->m_client_trid << "</clTRID>" << endl;
	 }
	 
	 if(response->m_trans_id->m_server_trid != NULL) {
	    cerr << "<svTRID>" << *response->m_trans_id->m_server_trid << "</svTRID>" << endl;
	 }
	 cerr << "</trID>\n";
      }
      cerr << "</response>\n";
      
   } // if "response"
   
} // printResponsePart2


bool handleArgs(epp_Session & sess, int argc, char * argv[],
		string & username, string & password,
                const string & extra_usage)
{
    vector<string> extra_usage_vec;
    if ( ! extra_usage.empty() ) extra_usage_vec.push_back(extra_usage);
    return handleArgs(sess,argc,argv,username,password,extra_usage_vec);
}

bool handleArgs(epp_Session & sess, int argc, char * argv[],
		string & username, string & password,
                const vector<string> & extra_usage)
{
   // CHANGE TRANSPORT HERE:
   typedef epp_TransportSSL myTransport;
   
   // The transport handling in this function is a bit more
   // complex than it needs to be for a Registrar, but this
   // function needs to be capable of handling all types of
   // transport.
   typedef refcnt_ptr<myTransport> myTransport_ref;
   epp_TransportBase* transport = new myTransport;
   
   // Default user name and password
   username = "testlogin";
   password = "testpass";

   // Map that will contain all command line arguments
   typedef map<string, string> argsMap;
   argsMap commandArgs;

   // Check all arguments and fill in the above map if it
   // is valid for the current transport
   if(argc > 1) {
      
      if (!strcmp(argv[1], "-?")) {	 
	 // Return with a line showing allowed command line arguments
	 
         string progname = argv[0];
         string prognamespaces = progname;
         unsigned int i;
         for ( i = 0; i < progname.length(); i++ ) { prognamespaces.replace(i, 1, " "); }

	 cout << progname;
	 
	 if(transport->getType() != "epp_TransportIOStream") {
	    // Only relevant for connection-transports
	    cout << " -n=<servername> -p=<serverPort>";
	 }
	 
	 cout << " -l=<login> -s=<pass>";
	   
	 if(transport->getType() == "epp_TransportSSL") {
	    // Only relevant for SSL-transport
	    // <FIXME>Bug: -ca must include full path to file...</fixme>
            cout << endl;
	    cout << prognamespaces << " -c=<keycert-pem> [-cadir=<cacertdir> -ca=<cacert-pem>]";
	 }
	 
         if ( extra_usage.size() > 0 ) {
            for ( vector<string>::const_iterator i = extra_usage.begin();
                  i != extra_usage.end();
                  i++ ) {
            cout << endl;
            cout << prognamespaces << " " << *i;
            }
         }
         
         cout << endl;
	 return false;
      } // if "-?"

      // If not "-?", process the rest of the command line arguments
      for(int parm = 1; parm < argc; parm++)
	{
//	   cout << parm << ": " << argv[parm] << endl;
	   
	   if (argv[parm][0] != '-') {
	      break;
	   }

	   if(transport->getType() != "epp_TransportIOStream") {
	      // Only relevant for connection-transports
	      
	      if (!strncmp(argv[parm], "-n=", 3)) {
		 commandArgs["n"] = &argv[parm][3];
	      } // if "-n"
	      
	      if (!strncmp(argv[parm], "-p=", 3)) {
		 commandArgs["p"] = &argv[parm][3];
	      } // if "-p"
	   }
	   
	   if(transport->getType() == "epp_TransportSSL") {
	      // Only relevant for SSL-transport
	      
	      if (!strncmp(argv[parm], "-c=", 3)) {
		 commandArgs["c"] = &argv[parm][3];
	      } // if "-c"
	      
	      if (!strncmp(argv[parm], "-cadir=", 7)) {
		 commandArgs["cadir"] = &argv[parm][7];
	      } // if "-cadir"
	      
	      if (!strncmp(argv[parm], "-ca=", 4)) {
		 commandArgs["ca"] = &argv[parm][4];
	      } // if "-ca"
	   }
	   
	   if (!strncmp(argv[parm], "-l=", 3)) {
	      username = &argv[parm][3];
	   } // if "-l"

	   if (!strncmp(argv[parm], "-s=", 3)) {
	      password = &argv[parm][3];
	   } // if "-s"
	   
	}// for
      
   }

#if 0
   // Write out all accepted command line arguments
   for( map<string, string>::const_iterator i = commandArgs.begin();
       i != commandArgs.end(); ++i)
     {
	cout << i->first << " " << i->second << endl ;
     }
#endif

   // If transport is SSL, and we need to set SSL-specific elements, we have to
   // use the epp_TransportSSL constructor to set these. This is because the
   // SSL-specific set/get methods are not part of the transport base class, and
   // we want to be able to compile this file regardless of which transport to use.
   //
   // If you are always going to use SSL, you can use get/set methods directly.
   if(transport->getType() == "epp_TransportSSL" && (commandArgs["c"] != "" || commandArgs["ca"] != "")) {
//      cout << commandArgs["c"] << " " <<  commandArgs["ca"] << " " <<  commandArgs["cadir"] << endl;
      transport = new epp_TransportSSL(commandArgs["c"], commandArgs["ca"], commandArgs["cadir"]);
   }

   // Set server name, if applicable
   if(commandArgs["n"] != "") {
      transport->setServerName(commandArgs["n"]);
   }
   
   // Set server port, if applicable
   if(commandArgs["p"] != "") {
      transport->setServerPort(atol(commandArgs["p"].c_str()));
   }
   
   // Put the transport into the session class
   myTransport_ref mt(dynamic_cast<myTransport*>(transport));
   sess.setTransport(mt);
   
   // Do not delete the transport pointer,
   // the ref-counter class takes care of it...
   // delete transport;
   
   // Finished
   return true;
   
} // handleArgs


void testLogIn(epp_Session & mySession, const string & username, const string & password)
{

#ifdef TEST_IO_REQUEST2CERR
   // We do not want to login when using IOStream transport
   if(mySession.getTransport()->getType() == "epp_TransportIOStream") {
      return;
   }
#endif  
#ifdef TEST_IO_RESULT2CERR
   // We do not want to login when using IOStream transport
   if(mySession.getTransport()->getType() == "epp_TransportIOStream") {
      return;
   }
#endif

   try {

      // Initialize <login> request object and sub-elements
      epp_LoginReq_ref l_req(new epp_LoginReq());
      l_req->m_cmd.ref(new epp_Command());
      
      // Fill in credentials information
      l_req->m_options.ref(new epp_Options(epp_string("1.0"),
					   epp_string("en")));
      l_req->m_client_id.ref(new epp_string(username));
      l_req->m_password.ref(new epp_string(password));
      
      // Fill in client transaction ID
      l_req->m_cmd->m_client_trid.ref(new epp_trid("LogIn-1"));
      
      // Fill in services information
      l_req->m_services.ref(new epp_objuri_seq());
      l_req->m_services->push_back("urn:ietf:params:xml:ns:contact-1.0");
      l_req->m_services->push_back("urn:ietf:params:xml:ns:domain-1.0");
      l_req->m_services->push_back("urn:ietf:params:xml:ns:host-1.0");

      // Initialize <login> command object
      epp_Login_ref l_obj(new epp_Login());
      
      // Put the request data object into the command object
      l_obj->setRequestData(*l_req);
      
      // Connect to server and get the greeting.
      epp_Greeting_ref myGr = mySession.connectAndGetGreeting();
      
      // processAction will write the data object to the server, wait for
      // the response, parse the response, and put the data back into the
      // session object.
      mySession.processAction(l_obj);
      
      // Return the <login> response object
      epp_LoginRsp_ref l_rsp = l_obj->getResponseData();
      
      // Print response if the transport is IOStream
      if(mySession.getTransport()->getType() == "epp_TransportIOStream") {
	 printResponsePart(l_rsp->m_rsp);
      }
   }
   catch(const epptransport::epp_TrException & ex) {
      cout << "Exception: " << ex.getString() << endl;
      exit(1);
   }
   catch(const epp_XMLException & ex) {
      cout << "Exception: " << ex.getString() << endl;
      exit(1);
   }
   catch(const epp_Exception & ex) {
      cout << "Exception: " << endl;
#ifndef TEST_IO_REQUEST2CERR
      cerr << "<response>" << endl;
      printResultsSeq(ex.m_details);
      printTransID(ex.m_trans_id);
      cerr << "</response>" << endl;
#endif
      exit(1);
   }
   catch(...) {
      cout << "Not able to login...\n";
      exit(1);
   }
} // testLogIn


void printResDataHeaders(const string & object, const string & returntype)
{
   cerr << "<resData>" << endl;
   cerr << "<" << object << ":" << returntype << endl;
   cerr << "\txmlns:" << object << "=\"urn:ietf:params:xml:ns:" << object << "-1.0\"" << endl;
   cerr << "\txsi:schemaLocation=\"urn:ietf:params:xml:ns:" << object << "-1.0" << endl;
   cerr << "\t" << object << "-1.0.xsd\">" << endl;
} // printResDataHeaders


void printResDataFooter(const string & object, const string & returntype)
{
   cerr << "</" << object << ":" << returntype << ">" << endl;
   cerr << "</resData>" << endl;
} // printResDataFooter


map<unsigned int,string> getTestDataMap(const string & testdata)
{
   map<unsigned int,string> dataMap;
   
   cout << "The entire testdata string is ["<<testdata<<"]" << endl;
   if(testdata.length() > 0) {
      
      if(testdata.at(0) != '\"') {
	 cout << "Error: Invalid data..." << endl;
	 return dataMap;
      }
      
      unsigned int begin = 1;
      unsigned int end = 1;
      unsigned int argc = 1;
      while((end = testdata.find('\"', begin)) != string::npos) {
	 
	 string tmp = testdata.substr(begin,end-begin);
         cout << "The testdata string token ["<< tmp <<"]" << endl;
	 if(tmp != " ") {
	    dataMap[argc] = tmp;
	    argc++;
	 }
	 begin = end+1;
      } // while
   } // if "testdata"
   return dataMap;
   
} // getTestDataMap

// This is an alternative to getTestDataMap -- it uses ":"
// separated values instead of '\"' separated one.
map<string,string> getExtraDataMap(const string & extradata)
{
   map<string,string> dataMap;
   
   string tmpstring = extradata;
   string sep = ":";
   // how's this for a hack -- use ":" for the separator,
   // unless there are ~'s in the string, then use that
   // instead -- only because in the host examples, the IPV6
   // addresses contain ":" characters.
   if ( tmpstring.find('~') != string::npos ) sep = "~";
   cout << "The entire extradata string is ["<<tmpstring<<"]" << endl;
   if(tmpstring.length() > 0) {
      
      unsigned int end = 1;
      while((end = tmpstring.find(sep)) != string::npos) {
	 
	 string tmp = tmpstring.substr(0,end);
         cout << "The extradata string token ["<< tmp <<"]" << endl;
	 if( ! tmp.empty() ) {
         
            string key, value;
            unsigned int equal_sign = tmp.find('=');
            if ( equal_sign != string::npos ) {
               key = tmp.substr(0,equal_sign);
               value = tmp.substr(equal_sign+1,tmp.length());
               cout << "key: ["<<key<<"] value ["<<value<<"]\n";
	       dataMap[key] = value;
            }
	 }
         
         tmpstring.erase(0,end+1);
         cout << "what's leftover: ["<<tmpstring<<"]\n";
         
      } // while
      
      if ( ! tmpstring.empty() ) {
         string key, value;
         unsigned int equal_sign = tmpstring.find('=');
         if ( equal_sign != string::npos ) {
            key = tmpstring.substr(0,equal_sign);
            value = tmpstring.substr(equal_sign+1,tmpstring.length());
            cout << "key: ["<<key<<"] value ["<<value<<"]\n";
	    dataMap[key] = value;
         }
      }
      
   } // if "tmpstring"
   return dataMap;
   
} // getExtraDataMap
