/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

#ifndef __EPP_TRANSPORTCONN_H
#define __EPP_TRANSPORTCONN_H

/** @file epp_TransportConn.h
 *  @brief Definition of epp_TransportConn
 *  @author Asbjorn Steira Mikkelsen
 */

#include <string>
#include "epp_TransportBase.h"

namespace epptransport {
   
   /// Abstract class defining common methods and attributes used in
   /// connection-based transport classes
   /// @see epp_TransportBase for additional information
   class epp_TransportConn : public epp_TransportBase {
      
    protected:
      std::string m_serverName; /**< The name of the server to connect to */
      unsigned long m_serverPort; /**< The port of the server to connect to */
      
    public:
      
      /// Null constructor, setting epp_TransportBase#m_type
      epp_TransportConn()
	{
	   m_type = "epp_TransportConn";
	   m_serverName = "127.0.0.1";
	   m_serverPort = 3121;
	};
      
      /// Destructor
      virtual ~epp_TransportConn() {};
      
      /// Abstract function, connects to server using server name and server port
      /// @param serverName Name of server
      /// @param serverPort Port of server
      virtual void connect(const std::string & serverName,
			   const unsigned long & serverPort) = 0;
      
      /// Abstract function, connects to server
      virtual void connect() = 0;
      
      /// Abstract function, disconnects from server
      virtual void disconnect() = 0;
      
      /// Returning the IP address of the server
      /// @return IP address of server
      /// @deprecated Use getServerName() instead
      virtual std::string getServerIP()
	{
	   return m_serverName;
	};
      
      /// Setting the IP address of the server
      /// @param serverIP IP address of server
      /// @deprecated Use setServerName() instead
      virtual void setServerIP(const std::string & serverIP)
	{
	   m_serverName = serverIP;
	}
      
      /// Returning the port of the server
      /// @return Port for server
      virtual unsigned long getServerPort()
	{
	   return m_serverPort;
	};
      
      /// Setting the port of the server
      /// @param serverPort Port for server
      virtual void setServerPort(const unsigned long & serverPort)
	{
	   m_serverPort = serverPort;
	}
      
      /// Returning the name of the server
      /// @return Name of server
      virtual std::string getServerName()
	{
	   return m_serverName;
	};
      
      /// Setting the name of the server
      /// @param serverName Name of server
      virtual void setServerName(const std::string & serverName)
	{
	   m_serverName = serverName;
	}

   };
   
   /// Typedef for the ref to the epp_TransportConn class
   typedef refcnt_ptr<epp_TransportConn> epp_TransportConn_ref;
}

#endif
