/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

#ifndef __EPP_TRANSPORTIOSTREAM_H
#define __EPP_TRANSPORTIOSTREAM_H

/** @file epp_TransportIOStream.h
 *  @brief Definition of epp_TransportIOStream
 *  @author Asbjorn Steira Mikkelsen
 */

#include <string>
#include "epp_TransportBase.h"

namespace epptransport {
   
   /// Class defining the IOStream transport, which reads from cin
   /// and writes to cout
   /// @see epp_TransportBase for additional information
   class epp_TransportIOStream : public epp_TransportBase {
      
    public:
      
      /// Null constructor, setting epp_TransportBase#m_type
      epp_TransportIOStream()
	{
	   m_type="epp_TransportIOStream";
	};
      
      /// Destructor
      virtual ~epp_TransportIOStream() {};
      
      /// Writing data to cout
      /// @param xml_string Data to write to cout
      void writeToServer(const std::string & xml_string);
      
      /// Reading data from cin
      /// @return Data read from cin
      std::string readFromServer();
   };
   
   /// Typedef for the ref to the epp_TransportIOStream class
   typedef refcnt_ptr<epp_TransportIOStream> epp_TransportIOStream_ref;
}

#endif
